from dapiclient.client import DAPIClient
import cbor2

client = DAPIClient()


def jsonrpc_tests():
    current_block_hash = client.getBestBlockHash()
    print('Test - getBestBlockHash: {}\n'.format(current_block_hash))

    height = 0
    genesis_block_hash = client.getBlockHash(height)
    print('Test - getBlockHash (height: {}): {}\n'.format(height, genesis_block_hash))

    mnlist_diff = client.getMnListDiff(genesis_block_hash, current_block_hash)
    print('Test - getMnListDiff:\n{}\n'.format(mnlist_diff))

    utxo = client.getUTXO('yPprxZrUL8UN73qLDS2xCg6yBFGieUWz7Q')#, 2)
    print('Test - getUTXO:\n{}\n'.format(utxo))

    address = 'yVs4HGmHgzP4t3gZ7KrpxRzCmkQcvZmczd'
    getAddressSummary = client.getAddressSummary(address)
    print('Test - getAddressSummary (address: {}): {}\n' .format(address, getAddressSummary))
    return mnlist_diff

def jsonrpc_comm_test(mnlist_diff):
    print('Test - Check for JSON-RPC response from all masternodes')
    for mn in mnlist_diff['mnList']:
        ip = mn['service'].split(':')[0]
        try:
            response = client.make_request_to_node('getBestBlockHash', {}, ip)
            print('Successful response from {}: {}'.format(ip, response))
        except Exception as ex:
            print('*** Failure from {} ***'.format(ip))


def main():
    mnlist_diff = jsonrpc_tests()

    jsonrpc_comm_test(mnlist_diff)


if __name__ == "__main__":
    main()



import sys
from dapiclient.MNDiscovery.masternode_discovery import MasternodeDiscovery
from dapiclient.rpc.jsonrpc.jsonrpc_client import JsonRpcClient
from dapiclient.rpc.grpc.grpc_client import GRpcClient


SEED_PORT = 3000
SEED_PORT_GRPC = 3010
GRPC_REQUEST_TIMEOUT = 5
GRPC_MAX_RESULTS = 100


class DAPIClient:
    """docstring for DAPIClient."""

    def __init__(self, seed_ip = None,mn_ip = None, options = {}):
        # support user-defined seeds and ip
        self.mn_ip = mn_ip
        self.mn_discovery = MasternodeDiscovery(seeds=seed_ip) #options['seeds'], options['port'])
        self.dapi_port = SEED_PORT #if options.port not in options else options.port
        self.native_grpc_port = SEED_PORT_GRPC #options.nativeGrpcPort || config.grpc.nativePort;
        self.timeout = 2000 #options.timeout || 2000;
        #self.forceJsonRpc = options.forceJsonRpc;
        #preconditionsUtil.checkArgument(jsutil.isUnsignedInteger(self.timeout),
        #  'Expect timeout to be an unsigned integer');
        self.retries = 1 #options.retries ? options.retries : 3;
        #preconditionsUtil.checkArgument(jsutil.isUnsignedInteger(self.retries),
        #  'Expect retries to be an unsigned integer');
        #self.dpp = new DPP();
        #self.make_request = {}
        #self.make_request['call_count'] = 0

    def make_request_to_random_dapi_node(self, method, params = {}, excluded_ips = []):
        #self.make_request['call_count'] = 0;

        return self.make_request_with_retries(method, params, self.retries, excluded_ips);


    def make_request(self, method, params, excluded_ips = []):
        #print(method)
        #self.make_request['call_count'] += 1
        random_masternode = self.mn_discovery.get_random_masternode(excluded_ips)
        #print(random_masternode)
        return JsonRpcClient.request({
          'host': random_masternode.split(':')[0],
          'port': self.dapi_port,
        }, method, params, { 'timeout': self.timeout })


    def make_request_to_node(self, method, params, ip):
        return JsonRpcClient.request({
          'host': ip,
          'port': self.dapi_port,
        }, method, params, { 'timeout': self.timeout })


    def make_request_with_retries(self, method, params, retries_count, excluded_ips):
        try:
            return self.make_request(method, params, excluded_ips)
        except Exception as ex:
            print('Exception:\n{}'.format(ex))
            #if (err.code !== 'ECONNABORTED' && err.code !== 'ECONNREFUSED') {
            #throw err;
          #}
            if (retries_count > 0):
                print('*** Retrying ***')
                excluded_on_next_try = []
                #if (err.address) {
                #  excludedOnNextTry = Array.isArray(excludedIps)
                #    ? excludedIps.slice().push(err.address) : excludedOnNextTry.push(err.address);
                #}
                return self.make_request_with_retries(method, params, retries_count - 1, excluded_on_next_try)

            raise Exception('max retries to connect to DAPI node reached');

    def getAddressSummary(self, address):
        return self.make_request_to_random_dapi_node('getAddressSummary', {'address': [address]})

    def getBestBlockHash(self):
        return self.make_request_to_random_dapi_node('getBestBlockHash')


    def getBlockHash(self, height):
        return self.make_request_to_random_dapi_node('getBlockHash', {'height': height})


    def getMnListDiff(self, baseBlockHash, blockHash):
        return self.make_request_to_random_dapi_node('getMnListDiff',
            {
                'baseBlockHash': baseBlockHash,
                'blockHash': blockHash
            })


    def getUTXO(self, address, from_result=0, to_result=1000, fromHeight=0, toHeight=sys.maxsize):
        return self.make_request_to_random_dapi_node('getUTXO',
            {
                'address': address,
                'from': from_result,
                'to': to_result,
                'fromHeight': fromHeight,
                'toHeight': toHeight
            })

    # gRPC endpoints

    def make_request_to_random_dapi_grpc_node(self, method, params = {}, options = {}, excluded_ips = []):
        #self.make_request['call_count'] = 0;
        if (self.mn_ip is None):
             random_masternode = self.mn_discovery.get_random_masternode()
             ip = random_masternode.split(":")[0]
        else:
             ip = self.mn_ip
        socket = '{}:{}'.format(ip, self.native_grpc_port)
        options = {
            'timeout': GRPC_REQUEST_TIMEOUT
        }

        return GRpcClient.request(socket, method, params, options)

    def subscribeToTransactionsWithProofs(self, bloom_filter, from_block_hash, from_block_height, count=0, send_transaction_hashes=0):
        return self.make_request_to_random_dapi_grpc_node(
                'subscribeToTransactionsWithProofs',
                {
                    'bloom_filter': bloom_filter,
                    'from_block_hash': from_block_hash,
                    'from_block_height': from_block_height,
                    'count': count,
                    'send_transaction_hashes': send_transaction_hashes
                }
        )

    def getBlock(self, hash=0, height=0):
        return self.make_request_to_random_dapi_grpc_node(
                'getBlock',
                {
                    'hash': hash,
                    'height': height
                }
            )

    def getStatus(self):
        return self.make_request_to_random_dapi_grpc_node(
                'getStatus'
        )

    def getTransaction(self, id):
        return self.make_request_to_random_dapi_grpc_node(
                'getTransaction',
                {
                    'id': id
                }
        )

    def sendTransaction(self, transaction, allow_high_fees=0, bypass_limits=0):
        return self.make_request_to_random_dapi_grpc_node(
                'sendTransaction',
                {
                    'transaction': transaction,
                    'allow_high_fees': allow_high_fees,
                    'bypass_limits': bypass_limits
                }
        )

    def getIdentity(self, id):
        return self.make_request_to_random_dapi_grpc_node(
                'getIdentity',
                {
                    'id': id
                }
            )

    def getDataContract(self, id):
        return self.make_request_to_random_dapi_grpc_node(
                'getDataContract',
                {
                    'id': id
                }
            )

    def getDocuments(self, data_contract_id, document_type, where=b'', order_by=b'', limit=GRPC_MAX_RESULTS, start_at=0, start_after=0):
        return self.make_request_to_random_dapi_grpc_node(
                'getDocuments',
                {
                    'data_contract_id': data_contract_id,
                    'document_type': document_type,
                    'where': where,
                    'order_by': order_by,
                    'limit': limit,
                    'start_at': start_at,
                    'start_after': start_after
                }
            )

    def applyStateTransition(self, state_transition):
        return self.make_request_to_random_dapi_grpc_node(
                'applyStateTransition',
                {
                    'state_transition': state_transition
                }
            )

    def getIdentityByFirstPublicKey(self, public_key_hash):
        return self.make_request_to_random_dapi_grpc_node(
                'getIdentityByFirstPublicKey',
                {
                    'public_key_hash': public_key_hash
                }
            )

    def getIdentityIdByFirstPublicKey(self, public_key_hash):
        return self.make_request_to_random_dapi_grpc_node(
                'getIdentityIdByFirstPublicKey',
                {
                    'public_key_hash': public_key_hash
                }
            )

def main():
    client = DAPIClient()
    return

if __name__ == "__main__":
    main()



import json
import time
from dapiclient.rpc.jsonrpc.jsonrpc_client import JsonRpcClient
from dapiclient.MNDiscovery.masternode_discovery import MasternodeDiscovery

def main():
    mnd = MasternodeDiscovery()
    print('Test - Get MN List')
    mnl = mnd.get_mnlist()
    print('Result: {} masternodes returned'.format(len(mnl)))

    print('Test - Random Node: {}'.format(mnd.get_random_masternode()))

    print('Test - Reset')
    mnd.reset()

    print('Test - Get MN List after reset')
    mnd.get_mnlist()

    print('Test - Get MN List before timeout')
    time.sleep(1)
    mnd.get_mnlist()

    print('Test - Try connecting to JSON-RPC on a MN')
    mnd.check_mn_response(mnl[0]['service'].split(':')[0])

    print('Test - Refresh list after timeout (60 seconds)')
    time.sleep(60)
    mnd.get_mnlist()
    print('Test - Complete')
    #print(json.dumps(mnl, indent = 2))

if __name__ == "__main__":
    main()



# Generated by the protocol buffer compiler.  DO NOT EDIT!
# source: platform.proto

import sys
_b=sys.version_info[0]<3 and (lambda x:x) or (lambda x:x.encode('latin1'))
from google.protobuf import descriptor as _descriptor
from google.protobuf import message as _message
from google.protobuf import reflection as _reflection
from google.protobuf import symbol_database as _symbol_database
from google.protobuf import descriptor_pb2
# @@protoc_insertion_point(imports)

_sym_db = _symbol_database.Default()




DESCRIPTOR = _descriptor.FileDescriptor(
  name='platform.proto',
  package='org.dash.platform.dapi.v0',
  syntax='proto3',
  serialized_pb=_b('\n\x0eplatform.proto\x12\x19org.dash.platform.dapi.v0\"\x93\x01\n\x0fStoreTreeProofs\x12\x18\n\x10identities_proof\x18\x01 \x01(\x0c\x12/\n\'public_key_hashes_to_identity_ids_proof\x18\x02 \x01(\x0c\x12\x1c\n\x14\x64\x61ta_contracts_proof\x18\x03 \x01(\x0c\x12\x17\n\x0f\x64ocuments_proof\x18\x04 \x01(\x0c\"\x97\x01\n\x05Proof\x12\x17\n\x0froot_tree_proof\x18\x01 \x01(\x0c\x12\x45\n\x11store_tree_proofs\x18\x02 \x01(\x0b\x32*.org.dash.platform.dapi.v0.StoreTreeProofs\x12\x1b\n\x13signature_llmq_hash\x18\x03 \x01(\x0c\x12\x11\n\tsignature\x18\x04 \x01(\x0c\"D\n\x10ResponseMetadata\x12\x0e\n\x06height\x18\x01 \x01(\x03\x12 \n\x18\x63ore_chain_locked_height\x18\x02 \x01(\r\"L\n\x1dStateTransitionBroadcastError\x12\x0c\n\x04\x63ode\x18\x01 \x01(\r\x12\x0f\n\x07message\x18\x02 \x01(\t\x12\x0c\n\x04\x64\x61ta\x18\x03 \x01(\x0c\";\n\x1f\x42roadcastStateTransitionRequest\x12\x18\n\x10state_transition\x18\x01 \x01(\x0c\"\"\n BroadcastStateTransitionResponse\"/\n\x12GetIdentityRequest\x12\n\n\x02id\x18\x01 \x01(\x0c\x12\r\n\x05prove\x18\x02 \x01(\x08\"\x97\x01\n\x13GetIdentityResponse\x12\x10\n\x08identity\x18\x01 \x01(\x0c\x12/\n\x05proof\x18\x02 \x01(\x0b\x32 .org.dash.platform.dapi.v0.Proof\x12=\n\x08metadata\x18\x03 \x01(\x0b\x32+.org.dash.platform.dapi.v0.ResponseMetadata\"3\n\x16GetDataContractRequest\x12\n\n\x02id\x18\x01 \x01(\x0c\x12\r\n\x05prove\x18\x02 \x01(\x08\"\xa0\x01\n\x17GetDataContractResponse\x12\x15\n\rdata_contract\x18\x01 \x01(\x0c\x12/\n\x05proof\x18\x02 \x01(\x0b\x32 .org.dash.platform.dapi.v0.Proof\x12=\n\x08metadata\x18\x03 \x01(\x0b\x32+.org.dash.platform.dapi.v0.ResponseMetadata\"\xb9\x01\n\x13GetDocumentsRequest\x12\x18\n\x10\x64\x61ta_contract_id\x18\x01 \x01(\x0c\x12\x15\n\rdocument_type\x18\x02 \x01(\t\x12\r\n\x05where\x18\x03 \x01(\x0c\x12\x10\n\x08order_by\x18\x04 \x01(\x0c\x12\r\n\x05limit\x18\x05 \x01(\r\x12\x15\n\x0bstart_after\x18\x06 \x01(\rH\x00\x12\x12\n\x08start_at\x18\x07 \x01(\rH\x00\x12\r\n\x05prove\x18\x08 \x01(\x08\x42\x07\n\x05start\"\x99\x01\n\x14GetDocumentsResponse\x12\x11\n\tdocuments\x18\x01 \x03(\x0c\x12/\n\x05proof\x18\x02 \x01(\x0b\x32 .org.dash.platform.dapi.v0.Proof\x12=\n\x08metadata\x18\x03 \x01(\x0b\x32+.org.dash.platform.dapi.v0.ResponseMetadata\"Q\n%GetIdentitiesByPublicKeyHashesRequest\x12\x19\n\x11public_key_hashes\x18\x01 \x03(\x0c\x12\r\n\x05prove\x18\x02 \x01(\x08\"\xac\x01\n&GetIdentitiesByPublicKeyHashesResponse\x12\x12\n\nidentities\x18\x01 \x03(\x0c\x12/\n\x05proof\x18\x02 \x01(\x0b\x32 .org.dash.platform.dapi.v0.Proof\x12=\n\x08metadata\x18\x03 \x01(\x0b\x32+.org.dash.platform.dapi.v0.ResponseMetadata\"R\n&GetIdentityIdsByPublicKeyHashesRequest\x12\x19\n\x11public_key_hashes\x18\x01 \x03(\x0c\x12\r\n\x05prove\x18\x02 \x01(\x08\"\xaf\x01\n\'GetIdentityIdsByPublicKeyHashesResponse\x12\x14\n\x0cidentity_ids\x18\x01 \x03(\x0c\x12/\n\x05proof\x18\x02 \x01(\x0b\x32 .org.dash.platform.dapi.v0.Proof\x12=\n\x08metadata\x18\x03 \x01(\x0b\x32+.org.dash.platform.dapi.v0.ResponseMetadata\"S\n#WaitForStateTransitionResultRequest\x12\x1d\n\x15state_transition_hash\x18\x01 \x01(\x0c\x12\r\n\x05prove\x18\x02 \x01(\x08\"\xf0\x01\n$WaitForStateTransitionResultResponse\x12I\n\x05\x65rror\x18\x01 \x01(\x0b\x32\x38.org.dash.platform.dapi.v0.StateTransitionBroadcastErrorH\x00\x12\x31\n\x05proof\x18\x02 \x01(\x0b\x32 .org.dash.platform.dapi.v0.ProofH\x00\x12=\n\x08metadata\x18\x03 \x01(\x0b\x32+.org.dash.platform.dapi.v0.ResponseMetadataB\x0b\n\tresponses\"P\n\x14\x43onsensusParamsBlock\x12\x11\n\tmax_bytes\x18\x01 \x01(\t\x12\x0f\n\x07max_gas\x18\x02 \x01(\t\x12\x14\n\x0ctime_iota_ms\x18\x03 \x01(\t\"b\n\x17\x43onsensusParamsEvidence\x12\x1a\n\x12max_age_num_blocks\x18\x01 \x01(\t\x12\x18\n\x10max_age_duration\x18\x02 \x01(\t\x12\x11\n\tmax_bytes\x18\x03 \x01(\t\":\n\x19GetConsensusParamsRequest\x12\x0e\n\x06height\x18\x01 \x01(\x03\x12\r\n\x05prove\x18\x02 \x01(\x08\"\xa2\x01\n\x1aGetConsensusParamsResponse\x12>\n\x05\x62lock\x18\x01 \x01(\x0b\x32/.org.dash.platform.dapi.v0.ConsensusParamsBlock\x12\x44\n\x08\x65vidence\x18\x02 \x01(\x0b\x32\x32.org.dash.platform.dapi.v0.ConsensusParamsEvidence2\xf2\x08\n\x08Platform\x12\x93\x01\n\x18\x62roadcastStateTransition\x12:.org.dash.platform.dapi.v0.BroadcastStateTransitionRequest\x1a;.org.dash.platform.dapi.v0.BroadcastStateTransitionResponse\x12l\n\x0bgetIdentity\x12-.org.dash.platform.dapi.v0.GetIdentityRequest\x1a..org.dash.platform.dapi.v0.GetIdentityResponse\x12x\n\x0fgetDataContract\x12\x31.org.dash.platform.dapi.v0.GetDataContractRequest\x1a\x32.org.dash.platform.dapi.v0.GetDataContractResponse\x12o\n\x0cgetDocuments\x12..org.dash.platform.dapi.v0.GetDocumentsRequest\x1a/.org.dash.platform.dapi.v0.GetDocumentsResponse\x12\xa5\x01\n\x1egetIdentitiesByPublicKeyHashes\x12@.org.dash.platform.dapi.v0.GetIdentitiesByPublicKeyHashesRequest\x1a\x41.org.dash.platform.dapi.v0.GetIdentitiesByPublicKeyHashesResponse\x12\xa8\x01\n\x1fgetIdentityIdsByPublicKeyHashes\x12\x41.org.dash.platform.dapi.v0.GetIdentityIdsByPublicKeyHashesRequest\x1a\x42.org.dash.platform.dapi.v0.GetIdentityIdsByPublicKeyHashesResponse\x12\x9f\x01\n\x1cwaitForStateTransitionResult\x12>.org.dash.platform.dapi.v0.WaitForStateTransitionResultRequest\x1a?.org.dash.platform.dapi.v0.WaitForStateTransitionResultResponse\x12\x81\x01\n\x12getConsensusParams\x12\x34.org.dash.platform.dapi.v0.GetConsensusParamsRequest\x1a\x35.org.dash.platform.dapi.v0.GetConsensusParamsResponseb\x06proto3')
)




_STORETREEPROOFS = _descriptor.Descriptor(
  name='StoreTreeProofs',
  full_name='org.dash.platform.dapi.v0.StoreTreeProofs',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  fields=[
    _descriptor.FieldDescriptor(
      name='identities_proof', full_name='org.dash.platform.dapi.v0.StoreTreeProofs.identities_proof', index=0,
      number=1, type=12, cpp_type=9, label=1,
      has_default_value=False, default_value=_b(""),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None),
    _descriptor.FieldDescriptor(
      name='public_key_hashes_to_identity_ids_proof', full_name='org.dash.platform.dapi.v0.StoreTreeProofs.public_key_hashes_to_identity_ids_proof', index=1,
      number=2, type=12, cpp_type=9, label=1,
      has_default_value=False, default_value=_b(""),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None),
    _descriptor.FieldDescriptor(
      name='data_contracts_proof', full_name='org.dash.platform.dapi.v0.StoreTreeProofs.data_contracts_proof', index=2,
      number=3, type=12, cpp_type=9, label=1,
      has_default_value=False, default_value=_b(""),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None),
    _descriptor.FieldDescriptor(
      name='documents_proof', full_name='org.dash.platform.dapi.v0.StoreTreeProofs.documents_proof', index=3,
      number=4, type=12, cpp_type=9, label=1,
      has_default_value=False, default_value=_b(""),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=46,
  serialized_end=193,
)


_PROOF = _descriptor.Descriptor(
  name='Proof',
  full_name='org.dash.platform.dapi.v0.Proof',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  fields=[
    _descriptor.FieldDescriptor(
      name='root_tree_proof', full_name='org.dash.platform.dapi.v0.Proof.root_tree_proof', index=0,
      number=1, type=12, cpp_type=9, label=1,
      has_default_value=False, default_value=_b(""),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None),
    _descriptor.FieldDescriptor(
      name='store_tree_proofs', full_name='org.dash.platform.dapi.v0.Proof.store_tree_proofs', index=1,
      number=2, type=11, cpp_type=10, label=1,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None),
    _descriptor.FieldDescriptor(
      name='signature_llmq_hash', full_name='org.dash.platform.dapi.v0.Proof.signature_llmq_hash', index=2,
      number=3, type=12, cpp_type=9, label=1,
      has_default_value=False, default_value=_b(""),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None),
    _descriptor.FieldDescriptor(
      name='signature', full_name='org.dash.platform.dapi.v0.Proof.signature', index=3,
      number=4, type=12, cpp_type=9, label=1,
      has_default_value=False, default_value=_b(""),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=196,
  serialized_end=347,
)


_RESPONSEMETADATA = _descriptor.Descriptor(
  name='ResponseMetadata',
  full_name='org.dash.platform.dapi.v0.ResponseMetadata',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  fields=[
    _descriptor.FieldDescriptor(
      name='height', full_name='org.dash.platform.dapi.v0.ResponseMetadata.height', index=0,
      number=1, type=3, cpp_type=2, label=1,
      has_default_value=False, default_value=0,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None),
    _descriptor.FieldDescriptor(
      name='core_chain_locked_height', full_name='org.dash.platform.dapi.v0.ResponseMetadata.core_chain_locked_height', index=1,
      number=2, type=13, cpp_type=3, label=1,
      has_default_value=False, default_value=0,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=349,
  serialized_end=417,
)


_STATETRANSITIONBROADCASTERROR = _descriptor.Descriptor(
  name='StateTransitionBroadcastError',
  full_name='org.dash.platform.dapi.v0.StateTransitionBroadcastError',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  fields=[
    _descriptor.FieldDescriptor(
      name='code', full_name='org.dash.platform.dapi.v0.StateTransitionBroadcastError.code', index=0,
      number=1, type=13, cpp_type=3, label=1,
      has_default_value=False, default_value=0,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None),
    _descriptor.FieldDescriptor(
      name='message', full_name='org.dash.platform.dapi.v0.StateTransitionBroadcastError.message', index=1,
      number=2, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=_b("").decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None),
    _descriptor.FieldDescriptor(
      name='data', full_name='org.dash.platform.dapi.v0.StateTransitionBroadcastError.data', index=2,
      number=3, type=12, cpp_type=9, label=1,
      has_default_value=False, default_value=_b(""),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=419,
  serialized_end=495,
)


_BROADCASTSTATETRANSITIONREQUEST = _descriptor.Descriptor(
  name='BroadcastStateTransitionRequest',
  full_name='org.dash.platform.dapi.v0.BroadcastStateTransitionRequest',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  fields=[
    _descriptor.FieldDescriptor(
      name='state_transition', full_name='org.dash.platform.dapi.v0.BroadcastStateTransitionRequest.state_transition', index=0,
      number=1, type=12, cpp_type=9, label=1,
      has_default_value=False, default_value=_b(""),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=497,
  serialized_end=556,
)


_BROADCASTSTATETRANSITIONRESPONSE = _descriptor.Descriptor(
  name='BroadcastStateTransitionResponse',
  full_name='org.dash.platform.dapi.v0.BroadcastStateTransitionResponse',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  fields=[
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=558,
  serialized_end=592,
)


_GETIDENTITYREQUEST = _descriptor.Descriptor(
  name='GetIdentityRequest',
  full_name='org.dash.platform.dapi.v0.GetIdentityRequest',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  fields=[
    _descriptor.FieldDescriptor(
      name='id', full_name='org.dash.platform.dapi.v0.GetIdentityRequest.id', index=0,
      number=1, type=12, cpp_type=9, label=1,
      has_default_value=False, default_value=_b(""),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None),
    _descriptor.FieldDescriptor(
      name='prove', full_name='org.dash.platform.dapi.v0.GetIdentityRequest.prove', index=1,
      number=2, type=8, cpp_type=7, label=1,
      has_default_value=False, default_value=False,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=594,
  serialized_end=641,
)


_GETIDENTITYRESPONSE = _descriptor.Descriptor(
  name='GetIdentityResponse',
  full_name='org.dash.platform.dapi.v0.GetIdentityResponse',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  fields=[
    _descriptor.FieldDescriptor(
      name='identity', full_name='org.dash.platform.dapi.v0.GetIdentityResponse.identity', index=0,
      number=1, type=12, cpp_type=9, label=1,
      has_default_value=False, default_value=_b(""),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None),
    _descriptor.FieldDescriptor(
      name='proof', full_name='org.dash.platform.dapi.v0.GetIdentityResponse.proof', index=1,
      number=2, type=11, cpp_type=10, label=1,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None),
    _descriptor.FieldDescriptor(
      name='metadata', full_name='org.dash.platform.dapi.v0.GetIdentityResponse.metadata', index=2,
      number=3, type=11, cpp_type=10, label=1,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=644,
  serialized_end=795,
)


_GETDATACONTRACTREQUEST = _descriptor.Descriptor(
  name='GetDataContractRequest',
  full_name='org.dash.platform.dapi.v0.GetDataContractRequest',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  fields=[
    _descriptor.FieldDescriptor(
      name='id', full_name='org.dash.platform.dapi.v0.GetDataContractRequest.id', index=0,
      number=1, type=12, cpp_type=9, label=1,
      has_default_value=False, default_value=_b(""),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None),
    _descriptor.FieldDescriptor(
      name='prove', full_name='org.dash.platform.dapi.v0.GetDataContractRequest.prove', index=1,
      number=2, type=8, cpp_type=7, label=1,
      has_default_value=False, default_value=False,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=797,
  serialized_end=848,
)


_GETDATACONTRACTRESPONSE = _descriptor.Descriptor(
  name='GetDataContractResponse',
  full_name='org.dash.platform.dapi.v0.GetDataContractResponse',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  fields=[
    _descriptor.FieldDescriptor(
      name='data_contract', full_name='org.dash.platform.dapi.v0.GetDataContractResponse.data_contract', index=0,
      number=1, type=12, cpp_type=9, label=1,
      has_default_value=False, default_value=_b(""),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None),
    _descriptor.FieldDescriptor(
      name='proof', full_name='org.dash.platform.dapi.v0.GetDataContractResponse.proof', index=1,
      number=2, type=11, cpp_type=10, label=1,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None),
    _descriptor.FieldDescriptor(
      name='metadata', full_name='org.dash.platform.dapi.v0.GetDataContractResponse.metadata', index=2,
      number=3, type=11, cpp_type=10, label=1,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=851,
  serialized_end=1011,
)


_GETDOCUMENTSREQUEST = _descriptor.Descriptor(
  name='GetDocumentsRequest',
  full_name='org.dash.platform.dapi.v0.GetDocumentsRequest',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  fields=[
    _descriptor.FieldDescriptor(
      name='data_contract_id', full_name='org.dash.platform.dapi.v0.GetDocumentsRequest.data_contract_id', index=0,
      number=1, type=12, cpp_type=9, label=1,
      has_default_value=False, default_value=_b(""),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None),
    _descriptor.FieldDescriptor(
      name='document_type', full_name='org.dash.platform.dapi.v0.GetDocumentsRequest.document_type', index=1,
      number=2, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=_b("").decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None),
    _descriptor.FieldDescriptor(
      name='where', full_name='org.dash.platform.dapi.v0.GetDocumentsRequest.where', index=2,
      number=3, type=12, cpp_type=9, label=1,
      has_default_value=False, default_value=_b(""),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None),
    _descriptor.FieldDescriptor(
      name='order_by', full_name='org.dash.platform.dapi.v0.GetDocumentsRequest.order_by', index=3,
      number=4, type=12, cpp_type=9, label=1,
      has_default_value=False, default_value=_b(""),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None),
    _descriptor.FieldDescriptor(
      name='limit', full_name='org.dash.platform.dapi.v0.GetDocumentsRequest.limit', index=4,
      number=5, type=13, cpp_type=3, label=1,
      has_default_value=False, default_value=0,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None),
    _descriptor.FieldDescriptor(
      name='start_after', full_name='org.dash.platform.dapi.v0.GetDocumentsRequest.start_after', index=5,
      number=6, type=13, cpp_type=3, label=1,
      has_default_value=False, default_value=0,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None),
    _descriptor.FieldDescriptor(
      name='start_at', full_name='org.dash.platform.dapi.v0.GetDocumentsRequest.start_at', index=6,
      number=7, type=13, cpp_type=3, label=1,
      has_default_value=False, default_value=0,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None),
    _descriptor.FieldDescriptor(
      name='prove', full_name='org.dash.platform.dapi.v0.GetDocumentsRequest.prove', index=7,
      number=8, type=8, cpp_type=7, label=1,
      has_default_value=False, default_value=False,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
    _descriptor.OneofDescriptor(
      name='start', full_name='org.dash.platform.dapi.v0.GetDocumentsRequest.start',
      index=0, containing_type=None, fields=[]),
  ],
  serialized_start=1014,
  serialized_end=1199,
)


_GETDOCUMENTSRESPONSE = _descriptor.Descriptor(
  name='GetDocumentsResponse',
  full_name='org.dash.platform.dapi.v0.GetDocumentsResponse',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  fields=[
    _descriptor.FieldDescriptor(
      name='documents', full_name='org.dash.platform.dapi.v0.GetDocumentsResponse.documents', index=0,
      number=1, type=12, cpp_type=9, label=3,
      has_default_value=False, default_value=[],
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None),
    _descriptor.FieldDescriptor(
      name='proof', full_name='org.dash.platform.dapi.v0.GetDocumentsResponse.proof', index=1,
      number=2, type=11, cpp_type=10, label=1,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None),
    _descriptor.FieldDescriptor(
      name='metadata', full_name='org.dash.platform.dapi.v0.GetDocumentsResponse.metadata', index=2,
      number=3, type=11, cpp_type=10, label=1,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=1202,
  serialized_end=1355,
)


_GETIDENTITIESBYPUBLICKEYHASHESREQUEST = _descriptor.Descriptor(
  name='GetIdentitiesByPublicKeyHashesRequest',
  full_name='org.dash.platform.dapi.v0.GetIdentitiesByPublicKeyHashesRequest',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  fields=[
    _descriptor.FieldDescriptor(
      name='public_key_hashes', full_name='org.dash.platform.dapi.v0.GetIdentitiesByPublicKeyHashesRequest.public_key_hashes', index=0,
      number=1, type=12, cpp_type=9, label=3,
      has_default_value=False, default_value=[],
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None),
    _descriptor.FieldDescriptor(
      name='prove', full_name='org.dash.platform.dapi.v0.GetIdentitiesByPublicKeyHashesRequest.prove', index=1,
      number=2, type=8, cpp_type=7, label=1,
      has_default_value=False, default_value=False,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=1357,
  serialized_end=1438,
)


_GETIDENTITIESBYPUBLICKEYHASHESRESPONSE = _descriptor.Descriptor(
  name='GetIdentitiesByPublicKeyHashesResponse',
  full_name='org.dash.platform.dapi.v0.GetIdentitiesByPublicKeyHashesResponse',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  fields=[
    _descriptor.FieldDescriptor(
      name='identities', full_name='org.dash.platform.dapi.v0.GetIdentitiesByPublicKeyHashesResponse.identities', index=0,
      number=1, type=12, cpp_type=9, label=3,
      has_default_value=False, default_value=[],
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None),
    _descriptor.FieldDescriptor(
      name='proof', full_name='org.dash.platform.dapi.v0.GetIdentitiesByPublicKeyHashesResponse.proof', index=1,
      number=2, type=11, cpp_type=10, label=1,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None),
    _descriptor.FieldDescriptor(
      name='metadata', full_name='org.dash.platform.dapi.v0.GetIdentitiesByPublicKeyHashesResponse.metadata', index=2,
      number=3, type=11, cpp_type=10, label=1,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=1441,
  serialized_end=1613,
)


_GETIDENTITYIDSBYPUBLICKEYHASHESREQUEST = _descriptor.Descriptor(
  name='GetIdentityIdsByPublicKeyHashesRequest',
  full_name='org.dash.platform.dapi.v0.GetIdentityIdsByPublicKeyHashesRequest',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  fields=[
    _descriptor.FieldDescriptor(
      name='public_key_hashes', full_name='org.dash.platform.dapi.v0.GetIdentityIdsByPublicKeyHashesRequest.public_key_hashes', index=0,
      number=1, type=12, cpp_type=9, label=3,
      has_default_value=False, default_value=[],
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None),
    _descriptor.FieldDescriptor(
      name='prove', full_name='org.dash.platform.dapi.v0.GetIdentityIdsByPublicKeyHashesRequest.prove', index=1,
      number=2, type=8, cpp_type=7, label=1,
      has_default_value=False, default_value=False,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=1615,
  serialized_end=1697,
)


_GETIDENTITYIDSBYPUBLICKEYHASHESRESPONSE = _descriptor.Descriptor(
  name='GetIdentityIdsByPublicKeyHashesResponse',
  full_name='org.dash.platform.dapi.v0.GetIdentityIdsByPublicKeyHashesResponse',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  fields=[
    _descriptor.FieldDescriptor(
      name='identity_ids', full_name='org.dash.platform.dapi.v0.GetIdentityIdsByPublicKeyHashesResponse.identity_ids', index=0,
      number=1, type=12, cpp_type=9, label=3,
      has_default_value=False, default_value=[],
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None),
    _descriptor.FieldDescriptor(
      name='proof', full_name='org.dash.platform.dapi.v0.GetIdentityIdsByPublicKeyHashesResponse.proof', index=1,
      number=2, type=11, cpp_type=10, label=1,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None),
    _descriptor.FieldDescriptor(
      name='metadata', full_name='org.dash.platform.dapi.v0.GetIdentityIdsByPublicKeyHashesResponse.metadata', index=2,
      number=3, type=11, cpp_type=10, label=1,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=1700,
  serialized_end=1875,
)


_WAITFORSTATETRANSITIONRESULTREQUEST = _descriptor.Descriptor(
  name='WaitForStateTransitionResultRequest',
  full_name='org.dash.platform.dapi.v0.WaitForStateTransitionResultRequest',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  fields=[
    _descriptor.FieldDescriptor(
      name='state_transition_hash', full_name='org.dash.platform.dapi.v0.WaitForStateTransitionResultRequest.state_transition_hash', index=0,
      number=1, type=12, cpp_type=9, label=1,
      has_default_value=False, default_value=_b(""),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None),
    _descriptor.FieldDescriptor(
      name='prove', full_name='org.dash.platform.dapi.v0.WaitForStateTransitionResultRequest.prove', index=1,
      number=2, type=8, cpp_type=7, label=1,
      has_default_value=False, default_value=False,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=1877,
  serialized_end=1960,
)


_WAITFORSTATETRANSITIONRESULTRESPONSE = _descriptor.Descriptor(
  name='WaitForStateTransitionResultResponse',
  full_name='org.dash.platform.dapi.v0.WaitForStateTransitionResultResponse',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  fields=[
    _descriptor.FieldDescriptor(
      name='error', full_name='org.dash.platform.dapi.v0.WaitForStateTransitionResultResponse.error', index=0,
      number=1, type=11, cpp_type=10, label=1,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None),
    _descriptor.FieldDescriptor(
      name='proof', full_name='org.dash.platform.dapi.v0.WaitForStateTransitionResultResponse.proof', index=1,
      number=2, type=11, cpp_type=10, label=1,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None),
    _descriptor.FieldDescriptor(
      name='metadata', full_name='org.dash.platform.dapi.v0.WaitForStateTransitionResultResponse.metadata', index=2,
      number=3, type=11, cpp_type=10, label=1,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
    _descriptor.OneofDescriptor(
      name='responses', full_name='org.dash.platform.dapi.v0.WaitForStateTransitionResultResponse.responses',
      index=0, containing_type=None, fields=[]),
  ],
  serialized_start=1963,
  serialized_end=2203,
)


_CONSENSUSPARAMSBLOCK = _descriptor.Descriptor(
  name='ConsensusParamsBlock',
  full_name='org.dash.platform.dapi.v0.ConsensusParamsBlock',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  fields=[
    _descriptor.FieldDescriptor(
      name='max_bytes', full_name='org.dash.platform.dapi.v0.ConsensusParamsBlock.max_bytes', index=0,
      number=1, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=_b("").decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None),
    _descriptor.FieldDescriptor(
      name='max_gas', full_name='org.dash.platform.dapi.v0.ConsensusParamsBlock.max_gas', index=1,
      number=2, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=_b("").decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None),
    _descriptor.FieldDescriptor(
      name='time_iota_ms', full_name='org.dash.platform.dapi.v0.ConsensusParamsBlock.time_iota_ms', index=2,
      number=3, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=_b("").decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=2205,
  serialized_end=2285,
)


_CONSENSUSPARAMSEVIDENCE = _descriptor.Descriptor(
  name='ConsensusParamsEvidence',
  full_name='org.dash.platform.dapi.v0.ConsensusParamsEvidence',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  fields=[
    _descriptor.FieldDescriptor(
      name='max_age_num_blocks', full_name='org.dash.platform.dapi.v0.ConsensusParamsEvidence.max_age_num_blocks', index=0,
      number=1, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=_b("").decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None),
    _descriptor.FieldDescriptor(
      name='max_age_duration', full_name='org.dash.platform.dapi.v0.ConsensusParamsEvidence.max_age_duration', index=1,
      number=2, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=_b("").decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None),
    _descriptor.FieldDescriptor(
      name='max_bytes', full_name='org.dash.platform.dapi.v0.ConsensusParamsEvidence.max_bytes', index=2,
      number=3, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=_b("").decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=2287,
  serialized_end=2385,
)


_GETCONSENSUSPARAMSREQUEST = _descriptor.Descriptor(
  name='GetConsensusParamsRequest',
  full_name='org.dash.platform.dapi.v0.GetConsensusParamsRequest',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  fields=[
    _descriptor.FieldDescriptor(
      name='height', full_name='org.dash.platform.dapi.v0.GetConsensusParamsRequest.height', index=0,
      number=1, type=3, cpp_type=2, label=1,
      has_default_value=False, default_value=0,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None),
    _descriptor.FieldDescriptor(
      name='prove', full_name='org.dash.platform.dapi.v0.GetConsensusParamsRequest.prove', index=1,
      number=2, type=8, cpp_type=7, label=1,
      has_default_value=False, default_value=False,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=2387,
  serialized_end=2445,
)


_GETCONSENSUSPARAMSRESPONSE = _descriptor.Descriptor(
  name='GetConsensusParamsResponse',
  full_name='org.dash.platform.dapi.v0.GetConsensusParamsResponse',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  fields=[
    _descriptor.FieldDescriptor(
      name='block', full_name='org.dash.platform.dapi.v0.GetConsensusParamsResponse.block', index=0,
      number=1, type=11, cpp_type=10, label=1,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None),
    _descriptor.FieldDescriptor(
      name='evidence', full_name='org.dash.platform.dapi.v0.GetConsensusParamsResponse.evidence', index=1,
      number=2, type=11, cpp_type=10, label=1,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=2448,
  serialized_end=2610,
)

_PROOF.fields_by_name['store_tree_proofs'].message_type = _STORETREEPROOFS
_GETIDENTITYRESPONSE.fields_by_name['proof'].message_type = _PROOF
_GETIDENTITYRESPONSE.fields_by_name['metadata'].message_type = _RESPONSEMETADATA
_GETDATACONTRACTRESPONSE.fields_by_name['proof'].message_type = _PROOF
_GETDATACONTRACTRESPONSE.fields_by_name['metadata'].message_type = _RESPONSEMETADATA
_GETDOCUMENTSREQUEST.oneofs_by_name['start'].fields.append(
  _GETDOCUMENTSREQUEST.fields_by_name['start_after'])
_GETDOCUMENTSREQUEST.fields_by_name['start_after'].containing_oneof = _GETDOCUMENTSREQUEST.oneofs_by_name['start']
_GETDOCUMENTSREQUEST.oneofs_by_name['start'].fields.append(
  _GETDOCUMENTSREQUEST.fields_by_name['start_at'])
_GETDOCUMENTSREQUEST.fields_by_name['start_at'].containing_oneof = _GETDOCUMENTSREQUEST.oneofs_by_name['start']
_GETDOCUMENTSRESPONSE.fields_by_name['proof'].message_type = _PROOF
_GETDOCUMENTSRESPONSE.fields_by_name['metadata'].message_type = _RESPONSEMETADATA
_GETIDENTITIESBYPUBLICKEYHASHESRESPONSE.fields_by_name['proof'].message_type = _PROOF
_GETIDENTITIESBYPUBLICKEYHASHESRESPONSE.fields_by_name['metadata'].message_type = _RESPONSEMETADATA
_GETIDENTITYIDSBYPUBLICKEYHASHESRESPONSE.fields_by_name['proof'].message_type = _PROOF
_GETIDENTITYIDSBYPUBLICKEYHASHESRESPONSE.fields_by_name['metadata'].message_type = _RESPONSEMETADATA
_WAITFORSTATETRANSITIONRESULTRESPONSE.fields_by_name['error'].message_type = _STATETRANSITIONBROADCASTERROR
_WAITFORSTATETRANSITIONRESULTRESPONSE.fields_by_name['proof'].message_type = _PROOF
_WAITFORSTATETRANSITIONRESULTRESPONSE.fields_by_name['metadata'].message_type = _RESPONSEMETADATA
_WAITFORSTATETRANSITIONRESULTRESPONSE.oneofs_by_name['responses'].fields.append(
  _WAITFORSTATETRANSITIONRESULTRESPONSE.fields_by_name['error'])
_WAITFORSTATETRANSITIONRESULTRESPONSE.fields_by_name['error'].containing_oneof = _WAITFORSTATETRANSITIONRESULTRESPONSE.oneofs_by_name['responses']
_WAITFORSTATETRANSITIONRESULTRESPONSE.oneofs_by_name['responses'].fields.append(
  _WAITFORSTATETRANSITIONRESULTRESPONSE.fields_by_name['proof'])
_WAITFORSTATETRANSITIONRESULTRESPONSE.fields_by_name['proof'].containing_oneof = _WAITFORSTATETRANSITIONRESULTRESPONSE.oneofs_by_name['responses']
_GETCONSENSUSPARAMSRESPONSE.fields_by_name['block'].message_type = _CONSENSUSPARAMSBLOCK
_GETCONSENSUSPARAMSRESPONSE.fields_by_name['evidence'].message_type = _CONSENSUSPARAMSEVIDENCE
DESCRIPTOR.message_types_by_name['StoreTreeProofs'] = _STORETREEPROOFS
DESCRIPTOR.message_types_by_name['Proof'] = _PROOF
DESCRIPTOR.message_types_by_name['ResponseMetadata'] = _RESPONSEMETADATA
DESCRIPTOR.message_types_by_name['StateTransitionBroadcastError'] = _STATETRANSITIONBROADCASTERROR
DESCRIPTOR.message_types_by_name['BroadcastStateTransitionRequest'] = _BROADCASTSTATETRANSITIONREQUEST
DESCRIPTOR.message_types_by_name['BroadcastStateTransitionResponse'] = _BROADCASTSTATETRANSITIONRESPONSE
DESCRIPTOR.message_types_by_name['GetIdentityRequest'] = _GETIDENTITYREQUEST
DESCRIPTOR.message_types_by_name['GetIdentityResponse'] = _GETIDENTITYRESPONSE
DESCRIPTOR.message_types_by_name['GetDataContractRequest'] = _GETDATACONTRACTREQUEST
DESCRIPTOR.message_types_by_name['GetDataContractResponse'] = _GETDATACONTRACTRESPONSE
DESCRIPTOR.message_types_by_name['GetDocumentsRequest'] = _GETDOCUMENTSREQUEST
DESCRIPTOR.message_types_by_name['GetDocumentsResponse'] = _GETDOCUMENTSRESPONSE
DESCRIPTOR.message_types_by_name['GetIdentitiesByPublicKeyHashesRequest'] = _GETIDENTITIESBYPUBLICKEYHASHESREQUEST
DESCRIPTOR.message_types_by_name['GetIdentitiesByPublicKeyHashesResponse'] = _GETIDENTITIESBYPUBLICKEYHASHESRESPONSE
DESCRIPTOR.message_types_by_name['GetIdentityIdsByPublicKeyHashesRequest'] = _GETIDENTITYIDSBYPUBLICKEYHASHESREQUEST
DESCRIPTOR.message_types_by_name['GetIdentityIdsByPublicKeyHashesResponse'] = _GETIDENTITYIDSBYPUBLICKEYHASHESRESPONSE
DESCRIPTOR.message_types_by_name['WaitForStateTransitionResultRequest'] = _WAITFORSTATETRANSITIONRESULTREQUEST
DESCRIPTOR.message_types_by_name['WaitForStateTransitionResultResponse'] = _WAITFORSTATETRANSITIONRESULTRESPONSE
DESCRIPTOR.message_types_by_name['ConsensusParamsBlock'] = _CONSENSUSPARAMSBLOCK
DESCRIPTOR.message_types_by_name['ConsensusParamsEvidence'] = _CONSENSUSPARAMSEVIDENCE
DESCRIPTOR.message_types_by_name['GetConsensusParamsRequest'] = _GETCONSENSUSPARAMSREQUEST
DESCRIPTOR.message_types_by_name['GetConsensusParamsResponse'] = _GETCONSENSUSPARAMSRESPONSE
_sym_db.RegisterFileDescriptor(DESCRIPTOR)

StoreTreeProofs = _reflection.GeneratedProtocolMessageType('StoreTreeProofs', (_message.Message,), dict(
  DESCRIPTOR = _STORETREEPROOFS,
  __module__ = 'platform_pb2'
  # @@protoc_insertion_point(class_scope:org.dash.platform.dapi.v0.StoreTreeProofs)
  ))
_sym_db.RegisterMessage(StoreTreeProofs)

Proof = _reflection.GeneratedProtocolMessageType('Proof', (_message.Message,), dict(
  DESCRIPTOR = _PROOF,
  __module__ = 'platform_pb2'
  # @@protoc_insertion_point(class_scope:org.dash.platform.dapi.v0.Proof)
  ))
_sym_db.RegisterMessage(Proof)

ResponseMetadata = _reflection.GeneratedProtocolMessageType('ResponseMetadata', (_message.Message,), dict(
  DESCRIPTOR = _RESPONSEMETADATA,
  __module__ = 'platform_pb2'
  # @@protoc_insertion_point(class_scope:org.dash.platform.dapi.v0.ResponseMetadata)
  ))
_sym_db.RegisterMessage(ResponseMetadata)

StateTransitionBroadcastError = _reflection.GeneratedProtocolMessageType('StateTransitionBroadcastError', (_message.Message,), dict(
  DESCRIPTOR = _STATETRANSITIONBROADCASTERROR,
  __module__ = 'platform_pb2'
  # @@protoc_insertion_point(class_scope:org.dash.platform.dapi.v0.StateTransitionBroadcastError)
  ))
_sym_db.RegisterMessage(StateTransitionBroadcastError)

BroadcastStateTransitionRequest = _reflection.GeneratedProtocolMessageType('BroadcastStateTransitionRequest', (_message.Message,), dict(
  DESCRIPTOR = _BROADCASTSTATETRANSITIONREQUEST,
  __module__ = 'platform_pb2'
  # @@protoc_insertion_point(class_scope:org.dash.platform.dapi.v0.BroadcastStateTransitionRequest)
  ))
_sym_db.RegisterMessage(BroadcastStateTransitionRequest)

BroadcastStateTransitionResponse = _reflection.GeneratedProtocolMessageType('BroadcastStateTransitionResponse', (_message.Message,), dict(
  DESCRIPTOR = _BROADCASTSTATETRANSITIONRESPONSE,
  __module__ = 'platform_pb2'
  # @@protoc_insertion_point(class_scope:org.dash.platform.dapi.v0.BroadcastStateTransitionResponse)
  ))
_sym_db.RegisterMessage(BroadcastStateTransitionResponse)

GetIdentityRequest = _reflection.GeneratedProtocolMessageType('GetIdentityRequest', (_message.Message,), dict(
  DESCRIPTOR = _GETIDENTITYREQUEST,
  __module__ = 'platform_pb2'
  # @@protoc_insertion_point(class_scope:org.dash.platform.dapi.v0.GetIdentityRequest)
  ))
_sym_db.RegisterMessage(GetIdentityRequest)

GetIdentityResponse = _reflection.GeneratedProtocolMessageType('GetIdentityResponse', (_message.Message,), dict(
  DESCRIPTOR = _GETIDENTITYRESPONSE,
  __module__ = 'platform_pb2'
  # @@protoc_insertion_point(class_scope:org.dash.platform.dapi.v0.GetIdentityResponse)
  ))
_sym_db.RegisterMessage(GetIdentityResponse)

GetDataContractRequest = _reflection.GeneratedProtocolMessageType('GetDataContractRequest', (_message.Message,), dict(
  DESCRIPTOR = _GETDATACONTRACTREQUEST,
  __module__ = 'platform_pb2'
  # @@protoc_insertion_point(class_scope:org.dash.platform.dapi.v0.GetDataContractRequest)
  ))
_sym_db.RegisterMessage(GetDataContractRequest)

GetDataContractResponse = _reflection.GeneratedProtocolMessageType('GetDataContractResponse', (_message.Message,), dict(
  DESCRIPTOR = _GETDATACONTRACTRESPONSE,
  __module__ = 'platform_pb2'
  # @@protoc_insertion_point(class_scope:org.dash.platform.dapi.v0.GetDataContractResponse)
  ))
_sym_db.RegisterMessage(GetDataContractResponse)

GetDocumentsRequest = _reflection.GeneratedProtocolMessageType('GetDocumentsRequest', (_message.Message,), dict(
  DESCRIPTOR = _GETDOCUMENTSREQUEST,
  __module__ = 'platform_pb2'
  # @@protoc_insertion_point(class_scope:org.dash.platform.dapi.v0.GetDocumentsRequest)
  ))
_sym_db.RegisterMessage(GetDocumentsRequest)

GetDocumentsResponse = _reflection.GeneratedProtocolMessageType('GetDocumentsResponse', (_message.Message,), dict(
  DESCRIPTOR = _GETDOCUMENTSRESPONSE,
  __module__ = 'platform_pb2'
  # @@protoc_insertion_point(class_scope:org.dash.platform.dapi.v0.GetDocumentsResponse)
  ))
_sym_db.RegisterMessage(GetDocumentsResponse)

GetIdentitiesByPublicKeyHashesRequest = _reflection.GeneratedProtocolMessageType('GetIdentitiesByPublicKeyHashesRequest', (_message.Message,), dict(
  DESCRIPTOR = _GETIDENTITIESBYPUBLICKEYHASHESREQUEST,
  __module__ = 'platform_pb2'
  # @@protoc_insertion_point(class_scope:org.dash.platform.dapi.v0.GetIdentitiesByPublicKeyHashesRequest)
  ))
_sym_db.RegisterMessage(GetIdentitiesByPublicKeyHashesRequest)

GetIdentitiesByPublicKeyHashesResponse = _reflection.GeneratedProtocolMessageType('GetIdentitiesByPublicKeyHashesResponse', (_message.Message,), dict(
  DESCRIPTOR = _GETIDENTITIESBYPUBLICKEYHASHESRESPONSE,
  __module__ = 'platform_pb2'
  # @@protoc_insertion_point(class_scope:org.dash.platform.dapi.v0.GetIdentitiesByPublicKeyHashesResponse)
  ))
_sym_db.RegisterMessage(GetIdentitiesByPublicKeyHashesResponse)

GetIdentityIdsByPublicKeyHashesRequest = _reflection.GeneratedProtocolMessageType('GetIdentityIdsByPublicKeyHashesRequest', (_message.Message,), dict(
  DESCRIPTOR = _GETIDENTITYIDSBYPUBLICKEYHASHESREQUEST,
  __module__ = 'platform_pb2'
  # @@protoc_insertion_point(class_scope:org.dash.platform.dapi.v0.GetIdentityIdsByPublicKeyHashesRequest)
  ))
_sym_db.RegisterMessage(GetIdentityIdsByPublicKeyHashesRequest)

GetIdentityIdsByPublicKeyHashesResponse = _reflection.GeneratedProtocolMessageType('GetIdentityIdsByPublicKeyHashesResponse', (_message.Message,), dict(
  DESCRIPTOR = _GETIDENTITYIDSBYPUBLICKEYHASHESRESPONSE,
  __module__ = 'platform_pb2'
  # @@protoc_insertion_point(class_scope:org.dash.platform.dapi.v0.GetIdentityIdsByPublicKeyHashesResponse)
  ))
_sym_db.RegisterMessage(GetIdentityIdsByPublicKeyHashesResponse)

WaitForStateTransitionResultRequest = _reflection.GeneratedProtocolMessageType('WaitForStateTransitionResultRequest', (_message.Message,), dict(
  DESCRIPTOR = _WAITFORSTATETRANSITIONRESULTREQUEST,
  __module__ = 'platform_pb2'
  # @@protoc_insertion_point(class_scope:org.dash.platform.dapi.v0.WaitForStateTransitionResultRequest)
  ))
_sym_db.RegisterMessage(WaitForStateTransitionResultRequest)

WaitForStateTransitionResultResponse = _reflection.GeneratedProtocolMessageType('WaitForStateTransitionResultResponse', (_message.Message,), dict(
  DESCRIPTOR = _WAITFORSTATETRANSITIONRESULTRESPONSE,
  __module__ = 'platform_pb2'
  # @@protoc_insertion_point(class_scope:org.dash.platform.dapi.v0.WaitForStateTransitionResultResponse)
  ))
_sym_db.RegisterMessage(WaitForStateTransitionResultResponse)

ConsensusParamsBlock = _reflection.GeneratedProtocolMessageType('ConsensusParamsBlock', (_message.Message,), dict(
  DESCRIPTOR = _CONSENSUSPARAMSBLOCK,
  __module__ = 'platform_pb2'
  # @@protoc_insertion_point(class_scope:org.dash.platform.dapi.v0.ConsensusParamsBlock)
  ))
_sym_db.RegisterMessage(ConsensusParamsBlock)

ConsensusParamsEvidence = _reflection.GeneratedProtocolMessageType('ConsensusParamsEvidence', (_message.Message,), dict(
  DESCRIPTOR = _CONSENSUSPARAMSEVIDENCE,
  __module__ = 'platform_pb2'
  # @@protoc_insertion_point(class_scope:org.dash.platform.dapi.v0.ConsensusParamsEvidence)
  ))
_sym_db.RegisterMessage(ConsensusParamsEvidence)

GetConsensusParamsRequest = _reflection.GeneratedProtocolMessageType('GetConsensusParamsRequest', (_message.Message,), dict(
  DESCRIPTOR = _GETCONSENSUSPARAMSREQUEST,
  __module__ = 'platform_pb2'
  # @@protoc_insertion_point(class_scope:org.dash.platform.dapi.v0.GetConsensusParamsRequest)
  ))
_sym_db.RegisterMessage(GetConsensusParamsRequest)

GetConsensusParamsResponse = _reflection.GeneratedProtocolMessageType('GetConsensusParamsResponse', (_message.Message,), dict(
  DESCRIPTOR = _GETCONSENSUSPARAMSRESPONSE,
  __module__ = 'platform_pb2'
  # @@protoc_insertion_point(class_scope:org.dash.platform.dapi.v0.GetConsensusParamsResponse)
  ))
_sym_db.RegisterMessage(GetConsensusParamsResponse)



_PLATFORM = _descriptor.ServiceDescriptor(
  name='Platform',
  full_name='org.dash.platform.dapi.v0.Platform',
  file=DESCRIPTOR,
  index=0,
  options=None,
  serialized_start=2613,
  serialized_end=3751,
  methods=[
  _descriptor.MethodDescriptor(
    name='broadcastStateTransition',
    full_name='org.dash.platform.dapi.v0.Platform.broadcastStateTransition',
    index=0,
    containing_service=None,
    input_type=_BROADCASTSTATETRANSITIONREQUEST,
    output_type=_BROADCASTSTATETRANSITIONRESPONSE,
    options=None,
  ),
  _descriptor.MethodDescriptor(
    name='getIdentity',
    full_name='org.dash.platform.dapi.v0.Platform.getIdentity',
    index=1,
    containing_service=None,
    input_type=_GETIDENTITYREQUEST,
    output_type=_GETIDENTITYRESPONSE,
    options=None,
  ),
  _descriptor.MethodDescriptor(
    name='getDataContract',
    full_name='org.dash.platform.dapi.v0.Platform.getDataContract',
    index=2,
    containing_service=None,
    input_type=_GETDATACONTRACTREQUEST,
    output_type=_GETDATACONTRACTRESPONSE,
    options=None,
  ),
  _descriptor.MethodDescriptor(
    name='getDocuments',
    full_name='org.dash.platform.dapi.v0.Platform.getDocuments',
    index=3,
    containing_service=None,
    input_type=_GETDOCUMENTSREQUEST,
    output_type=_GETDOCUMENTSRESPONSE,
    options=None,
  ),
  _descriptor.MethodDescriptor(
    name='getIdentitiesByPublicKeyHashes',
    full_name='org.dash.platform.dapi.v0.Platform.getIdentitiesByPublicKeyHashes',
    index=4,
    containing_service=None,
    input_type=_GETIDENTITIESBYPUBLICKEYHASHESREQUEST,
    output_type=_GETIDENTITIESBYPUBLICKEYHASHESRESPONSE,
    options=None,
  ),
  _descriptor.MethodDescriptor(
    name='getIdentityIdsByPublicKeyHashes',
    full_name='org.dash.platform.dapi.v0.Platform.getIdentityIdsByPublicKeyHashes',
    index=5,
    containing_service=None,
    input_type=_GETIDENTITYIDSBYPUBLICKEYHASHESREQUEST,
    output_type=_GETIDENTITYIDSBYPUBLICKEYHASHESRESPONSE,
    options=None,
  ),
  _descriptor.MethodDescriptor(
    name='waitForStateTransitionResult',
    full_name='org.dash.platform.dapi.v0.Platform.waitForStateTransitionResult',
    index=6,
    containing_service=None,
    input_type=_WAITFORSTATETRANSITIONRESULTREQUEST,
    output_type=_WAITFORSTATETRANSITIONRESULTRESPONSE,
    options=None,
  ),
  _descriptor.MethodDescriptor(
    name='getConsensusParams',
    full_name='org.dash.platform.dapi.v0.Platform.getConsensusParams',
    index=7,
    containing_service=None,
    input_type=_GETCONSENSUSPARAMSREQUEST,
    output_type=_GETCONSENSUSPARAMSRESPONSE,
    options=None,
  ),
])
_sym_db.RegisterServiceDescriptor(_PLATFORM)

DESCRIPTOR.services_by_name['Platform'] = _PLATFORM

try:
  # THESE ELEMENTS WILL BE DEPRECATED.
  # Please use the generated *_pb2_grpc.py files instead.
  import grpc
  from grpc.beta import implementations as beta_implementations
  from grpc.beta import interfaces as beta_interfaces
  from grpc.framework.common import cardinality
  from grpc.framework.interfaces.face import utilities as face_utilities


  class PlatformStub(object):
    # missing associated documentation comment in .proto file
    pass

    def __init__(self, channel):
      """Constructor.
      Args:
        channel: A grpc.Channel.
      """
      self.broadcastStateTransition = channel.unary_unary(
          '/org.dash.platform.dapi.v0.Platform/broadcastStateTransition',
          request_serializer=BroadcastStateTransitionRequest.SerializeToString,
          response_deserializer=BroadcastStateTransitionResponse.FromString,
          )
      self.getIdentity = channel.unary_unary(
          '/org.dash.platform.dapi.v0.Platform/getIdentity',
          request_serializer=GetIdentityRequest.SerializeToString,
          response_deserializer=GetIdentityResponse.FromString,
          )
      self.getDataContract = channel.unary_unary(
          '/org.dash.platform.dapi.v0.Platform/getDataContract',
          request_serializer=GetDataContractRequest.SerializeToString,
          response_deserializer=GetDataContractResponse.FromString,
          )
      self.getDocuments = channel.unary_unary(
          '/org.dash.platform.dapi.v0.Platform/getDocuments',
          request_serializer=GetDocumentsRequest.SerializeToString,
          response_deserializer=GetDocumentsResponse.FromString,
          )
      self.getIdentitiesByPublicKeyHashes = channel.unary_unary(
          '/org.dash.platform.dapi.v0.Platform/getIdentitiesByPublicKeyHashes',
          request_serializer=GetIdentitiesByPublicKeyHashesRequest.SerializeToString,
          response_deserializer=GetIdentitiesByPublicKeyHashesResponse.FromString,
          )
      self.getIdentityIdsByPublicKeyHashes = channel.unary_unary(
          '/org.dash.platform.dapi.v0.Platform/getIdentityIdsByPublicKeyHashes',
          request_serializer=GetIdentityIdsByPublicKeyHashesRequest.SerializeToString,
          response_deserializer=GetIdentityIdsByPublicKeyHashesResponse.FromString,
          )
      self.waitForStateTransitionResult = channel.unary_unary(
          '/org.dash.platform.dapi.v0.Platform/waitForStateTransitionResult',
          request_serializer=WaitForStateTransitionResultRequest.SerializeToString,
          response_deserializer=WaitForStateTransitionResultResponse.FromString,
          )
      self.getConsensusParams = channel.unary_unary(
          '/org.dash.platform.dapi.v0.Platform/getConsensusParams',
          request_serializer=GetConsensusParamsRequest.SerializeToString,
          response_deserializer=GetConsensusParamsResponse.FromString,
          )


  class PlatformServicer(object):
    # missing associated documentation comment in .proto file
    pass

    def broadcastStateTransition(self, request, context):
      # missing associated documentation comment in .proto file
      pass
      context.set_code(grpc.StatusCode.UNIMPLEMENTED)
      context.set_details('Method not implemented!')
      raise NotImplementedError('Method not implemented!')

    def getIdentity(self, request, context):
      # missing associated documentation comment in .proto file
      pass
      context.set_code(grpc.StatusCode.UNIMPLEMENTED)
      context.set_details('Method not implemented!')
      raise NotImplementedError('Method not implemented!')

    def getDataContract(self, request, context):
      # missing associated documentation comment in .proto file
      pass
      context.set_code(grpc.StatusCode.UNIMPLEMENTED)
      context.set_details('Method not implemented!')
      raise NotImplementedError('Method not implemented!')

    def getDocuments(self, request, context):
      # missing associated documentation comment in .proto file
      pass
      context.set_code(grpc.StatusCode.UNIMPLEMENTED)
      context.set_details('Method not implemented!')
      raise NotImplementedError('Method not implemented!')

    def getIdentitiesByPublicKeyHashes(self, request, context):
      # missing associated documentation comment in .proto file
      pass
      context.set_code(grpc.StatusCode.UNIMPLEMENTED)
      context.set_details('Method not implemented!')
      raise NotImplementedError('Method not implemented!')

    def getIdentityIdsByPublicKeyHashes(self, request, context):
      # missing associated documentation comment in .proto file
      pass
      context.set_code(grpc.StatusCode.UNIMPLEMENTED)
      context.set_details('Method not implemented!')
      raise NotImplementedError('Method not implemented!')

    def waitForStateTransitionResult(self, request, context):
      # missing associated documentation comment in .proto file
      pass
      context.set_code(grpc.StatusCode.UNIMPLEMENTED)
      context.set_details('Method not implemented!')
      raise NotImplementedError('Method not implemented!')

    def getConsensusParams(self, request, context):
      # missing associated documentation comment in .proto file
      pass
      context.set_code(grpc.StatusCode.UNIMPLEMENTED)
      context.set_details('Method not implemented!')
      raise NotImplementedError('Method not implemented!')


  def add_PlatformServicer_to_server(servicer, server):
    rpc_method_handlers = {
        'broadcastStateTransition': grpc.unary_unary_rpc_method_handler(
            servicer.broadcastStateTransition,
            request_deserializer=BroadcastStateTransitionRequest.FromString,
            response_serializer=BroadcastStateTransitionResponse.SerializeToString,
        ),
        'getIdentity': grpc.unary_unary_rpc_method_handler(
            servicer.getIdentity,
            request_deserializer=GetIdentityRequest.FromString,
            response_serializer=GetIdentityResponse.SerializeToString,
        ),
        'getDataContract': grpc.unary_unary_rpc_method_handler(
            servicer.getDataContract,
            request_deserializer=GetDataContractRequest.FromString,
            response_serializer=GetDataContractResponse.SerializeToString,
        ),
        'getDocuments': grpc.unary_unary_rpc_method_handler(
            servicer.getDocuments,
            request_deserializer=GetDocumentsRequest.FromString,
            response_serializer=GetDocumentsResponse.SerializeToString,
        ),
        'getIdentitiesByPublicKeyHashes': grpc.unary_unary_rpc_method_handler(
            servicer.getIdentitiesByPublicKeyHashes,
            request_deserializer=GetIdentitiesByPublicKeyHashesRequest.FromString,
            response_serializer=GetIdentitiesByPublicKeyHashesResponse.SerializeToString,
        ),
        'getIdentityIdsByPublicKeyHashes': grpc.unary_unary_rpc_method_handler(
            servicer.getIdentityIdsByPublicKeyHashes,
            request_deserializer=GetIdentityIdsByPublicKeyHashesRequest.FromString,
            response_serializer=GetIdentityIdsByPublicKeyHashesResponse.SerializeToString,
        ),
        'waitForStateTransitionResult': grpc.unary_unary_rpc_method_handler(
            servicer.waitForStateTransitionResult,
            request_deserializer=WaitForStateTransitionResultRequest.FromString,
            response_serializer=WaitForStateTransitionResultResponse.SerializeToString,
        ),
        'getConsensusParams': grpc.unary_unary_rpc_method_handler(
            servicer.getConsensusParams,
            request_deserializer=GetConsensusParamsRequest.FromString,
            response_serializer=GetConsensusParamsResponse.SerializeToString,
        ),
    }
    generic_handler = grpc.method_handlers_generic_handler(
        'org.dash.platform.dapi.v0.Platform', rpc_method_handlers)
    server.add_generic_rpc_handlers((generic_handler,))


  class BetaPlatformServicer(object):
    """The Beta API is deprecated for 0.15.0 and later.
    It is recommended to use the GA API (classes and functions in this
    file not marked beta) for all further purposes. This class was generated
    only to ease transition from grpcio<0.15.0 to grpcio>=0.15.0."""
    # missing associated documentation comment in .proto file
    pass
    def broadcastStateTransition(self, request, context):
      # missing associated documentation comment in .proto file
      pass
      context.code(beta_interfaces.StatusCode.UNIMPLEMENTED)
    def getIdentity(self, request, context):
      # missing associated documentation comment in .proto file
      pass
      context.code(beta_interfaces.StatusCode.UNIMPLEMENTED)
    def getDataContract(self, request, context):
      # missing associated documentation comment in .proto file
      pass
      context.code(beta_interfaces.StatusCode.UNIMPLEMENTED)
    def getDocuments(self, request, context):
      # missing associated documentation comment in .proto file
      pass
      context.code(beta_interfaces.StatusCode.UNIMPLEMENTED)
    def getIdentitiesByPublicKeyHashes(self, request, context):
      # missing associated documentation comment in .proto file
      pass
      context.code(beta_interfaces.StatusCode.UNIMPLEMENTED)
    def getIdentityIdsByPublicKeyHashes(self, request, context):
      # missing associated documentation comment in .proto file
      pass
      context.code(beta_interfaces.StatusCode.UNIMPLEMENTED)
    def waitForStateTransitionResult(self, request, context):
      # missing associated documentation comment in .proto file
      pass
      context.code(beta_interfaces.StatusCode.UNIMPLEMENTED)
    def getConsensusParams(self, request, context):
      # missing associated documentation comment in .proto file
      pass
      context.code(beta_interfaces.StatusCode.UNIMPLEMENTED)


  class BetaPlatformStub(object):
    """The Beta API is deprecated for 0.15.0 and later.
    It is recommended to use the GA API (classes and functions in this
    file not marked beta) for all further purposes. This class was generated
    only to ease transition from grpcio<0.15.0 to grpcio>=0.15.0."""
    # missing associated documentation comment in .proto file
    pass
    def broadcastStateTransition(self, request, timeout, metadata=None, with_call=False, protocol_options=None):
      # missing associated documentation comment in .proto file
      pass
      raise NotImplementedError()
    broadcastStateTransition.future = None
    def getIdentity(self, request, timeout, metadata=None, with_call=False, protocol_options=None):
      # missing associated documentation comment in .proto file
      pass
      raise NotImplementedError()
    getIdentity.future = None
    def getDataContract(self, request, timeout, metadata=None, with_call=False, protocol_options=None):
      # missing associated documentation comment in .proto file
      pass
      raise NotImplementedError()
    getDataContract.future = None
    def getDocuments(self, request, timeout, metadata=None, with_call=False, protocol_options=None):
      # missing associated documentation comment in .proto file
      pass
      raise NotImplementedError()
    getDocuments.future = None
    def getIdentitiesByPublicKeyHashes(self, request, timeout, metadata=None, with_call=False, protocol_options=None):
      # missing associated documentation comment in .proto file
      pass
      raise NotImplementedError()
    getIdentitiesByPublicKeyHashes.future = None
    def getIdentityIdsByPublicKeyHashes(self, request, timeout, metadata=None, with_call=False, protocol_options=None):
      # missing associated documentation comment in .proto file
      pass
      raise NotImplementedError()
    getIdentityIdsByPublicKeyHashes.future = None
    def waitForStateTransitionResult(self, request, timeout, metadata=None, with_call=False, protocol_options=None):
      # missing associated documentation comment in .proto file
      pass
      raise NotImplementedError()
    waitForStateTransitionResult.future = None
    def getConsensusParams(self, request, timeout, metadata=None, with_call=False, protocol_options=None):
      # missing associated documentation comment in .proto file
      pass
      raise NotImplementedError()
    getConsensusParams.future = None


  def beta_create_Platform_server(servicer, pool=None, pool_size=None, default_timeout=None, maximum_timeout=None):
    """The Beta API is deprecated for 0.15.0 and later.
    It is recommended to use the GA API (classes and functions in this
    file not marked beta) for all further purposes. This function was
    generated only to ease transition from grpcio<0.15.0 to grpcio>=0.15.0"""
    request_deserializers = {
      ('org.dash.platform.dapi.v0.Platform', 'broadcastStateTransition'): BroadcastStateTransitionRequest.FromString,
      ('org.dash.platform.dapi.v0.Platform', 'getConsensusParams'): GetConsensusParamsRequest.FromString,
      ('org.dash.platform.dapi.v0.Platform', 'getDataContract'): GetDataContractRequest.FromString,
      ('org.dash.platform.dapi.v0.Platform', 'getDocuments'): GetDocumentsRequest.FromString,
      ('org.dash.platform.dapi.v0.Platform', 'getIdentitiesByPublicKeyHashes'): GetIdentitiesByPublicKeyHashesRequest.FromString,
      ('org.dash.platform.dapi.v0.Platform', 'getIdentity'): GetIdentityRequest.FromString,
      ('org.dash.platform.dapi.v0.Platform', 'getIdentityIdsByPublicKeyHashes'): GetIdentityIdsByPublicKeyHashesRequest.FromString,
      ('org.dash.platform.dapi.v0.Platform', 'waitForStateTransitionResult'): WaitForStateTransitionResultRequest.FromString,
    }
    response_serializers = {
      ('org.dash.platform.dapi.v0.Platform', 'broadcastStateTransition'): BroadcastStateTransitionResponse.SerializeToString,
      ('org.dash.platform.dapi.v0.Platform', 'getConsensusParams'): GetConsensusParamsResponse.SerializeToString,
      ('org.dash.platform.dapi.v0.Platform', 'getDataContract'): GetDataContractResponse.SerializeToString,
      ('org.dash.platform.dapi.v0.Platform', 'getDocuments'): GetDocumentsResponse.SerializeToString,
      ('org.dash.platform.dapi.v0.Platform', 'getIdentitiesByPublicKeyHashes'): GetIdentitiesByPublicKeyHashesResponse.SerializeToString,
      ('org.dash.platform.dapi.v0.Platform', 'getIdentity'): GetIdentityResponse.SerializeToString,
      ('org.dash.platform.dapi.v0.Platform', 'getIdentityIdsByPublicKeyHashes'): GetIdentityIdsByPublicKeyHashesResponse.SerializeToString,
      ('org.dash.platform.dapi.v0.Platform', 'waitForStateTransitionResult'): WaitForStateTransitionResultResponse.SerializeToString,
    }
    method_implementations = {
      ('org.dash.platform.dapi.v0.Platform', 'broadcastStateTransition'): face_utilities.unary_unary_inline(servicer.broadcastStateTransition),
      ('org.dash.platform.dapi.v0.Platform', 'getConsensusParams'): face_utilities.unary_unary_inline(servicer.getConsensusParams),
      ('org.dash.platform.dapi.v0.Platform', 'getDataContract'): face_utilities.unary_unary_inline(servicer.getDataContract),
      ('org.dash.platform.dapi.v0.Platform', 'getDocuments'): face_utilities.unary_unary_inline(servicer.getDocuments),
      ('org.dash.platform.dapi.v0.Platform', 'getIdentitiesByPublicKeyHashes'): face_utilities.unary_unary_inline(servicer.getIdentitiesByPublicKeyHashes),
      ('org.dash.platform.dapi.v0.Platform', 'getIdentity'): face_utilities.unary_unary_inline(servicer.getIdentity),
      ('org.dash.platform.dapi.v0.Platform', 'getIdentityIdsByPublicKeyHashes'): face_utilities.unary_unary_inline(servicer.getIdentityIdsByPublicKeyHashes),
      ('org.dash.platform.dapi.v0.Platform', 'waitForStateTransitionResult'): face_utilities.unary_unary_inline(servicer.waitForStateTransitionResult),
    }
    server_options = beta_implementations.server_options(request_deserializers=request_deserializers, response_serializers=response_serializers, thread_pool=pool, thread_pool_size=pool_size, default_timeout=default_timeout, maximum_timeout=maximum_timeout)
    return beta_implementations.server(method_implementations, options=server_options)


  def beta_create_Platform_stub(channel, host=None, metadata_transformer=None, pool=None, pool_size=None):
    """The Beta API is deprecated for 0.15.0 and later.
    It is recommended to use the GA API (classes and functions in this
    file not marked beta) for all further purposes. This function was
    generated only to ease transition from grpcio<0.15.0 to grpcio>=0.15.0"""
    request_serializers = {
      ('org.dash.platform.dapi.v0.Platform', 'broadcastStateTransition'): BroadcastStateTransitionRequest.SerializeToString,
      ('org.dash.platform.dapi.v0.Platform', 'getConsensusParams'): GetConsensusParamsRequest.SerializeToString,
      ('org.dash.platform.dapi.v0.Platform', 'getDataContract'): GetDataContractRequest.SerializeToString,
      ('org.dash.platform.dapi.v0.Platform', 'getDocuments'): GetDocumentsRequest.SerializeToString,
      ('org.dash.platform.dapi.v0.Platform', 'getIdentitiesByPublicKeyHashes'): GetIdentitiesByPublicKeyHashesRequest.SerializeToString,
      ('org.dash.platform.dapi.v0.Platform', 'getIdentity'): GetIdentityRequest.SerializeToString,
      ('org.dash.platform.dapi.v0.Platform', 'getIdentityIdsByPublicKeyHashes'): GetIdentityIdsByPublicKeyHashesRequest.SerializeToString,
      ('org.dash.platform.dapi.v0.Platform', 'waitForStateTransitionResult'): WaitForStateTransitionResultRequest.SerializeToString,
    }
    response_deserializers = {
      ('org.dash.platform.dapi.v0.Platform', 'broadcastStateTransition'): BroadcastStateTransitionResponse.FromString,
      ('org.dash.platform.dapi.v0.Platform', 'getConsensusParams'): GetConsensusParamsResponse.FromString,
      ('org.dash.platform.dapi.v0.Platform', 'getDataContract'): GetDataContractResponse.FromString,
      ('org.dash.platform.dapi.v0.Platform', 'getDocuments'): GetDocumentsResponse.FromString,
      ('org.dash.platform.dapi.v0.Platform', 'getIdentitiesByPublicKeyHashes'): GetIdentitiesByPublicKeyHashesResponse.FromString,
      ('org.dash.platform.dapi.v0.Platform', 'getIdentity'): GetIdentityResponse.FromString,
      ('org.dash.platform.dapi.v0.Platform', 'getIdentityIdsByPublicKeyHashes'): GetIdentityIdsByPublicKeyHashesResponse.FromString,
      ('org.dash.platform.dapi.v0.Platform', 'waitForStateTransitionResult'): WaitForStateTransitionResultResponse.FromString,
    }
    cardinalities = {
      'broadcastStateTransition': cardinality.Cardinality.UNARY_UNARY,
      'getConsensusParams': cardinality.Cardinality.UNARY_UNARY,
      'getDataContract': cardinality.Cardinality.UNARY_UNARY,
      'getDocuments': cardinality.Cardinality.UNARY_UNARY,
      'getIdentitiesByPublicKeyHashes': cardinality.Cardinality.UNARY_UNARY,
      'getIdentity': cardinality.Cardinality.UNARY_UNARY,
      'getIdentityIdsByPublicKeyHashes': cardinality.Cardinality.UNARY_UNARY,
      'waitForStateTransitionResult': cardinality.Cardinality.UNARY_UNARY,
    }
    stub_options = beta_implementations.stub_options(host=host, metadata_transformer=metadata_transformer, request_serializers=request_serializers, response_deserializers=response_deserializers, thread_pool=pool, thread_pool_size=pool_size)
    return beta_implementations.dynamic_stub(channel, 'org.dash.platform.dapi.v0.Platform', cardinalities, options=stub_options)
except ImportError:
  pass
# @@protoc_insertion_point(module_scope)


# Generated by the protocol buffer compiler.  DO NOT EDIT!
# source: transactions_filter_stream.proto

import sys
_b=sys.version_info[0]<3 and (lambda x:x) or (lambda x:x.encode('latin1'))
from google.protobuf import descriptor as _descriptor
from google.protobuf import message as _message
from google.protobuf import reflection as _reflection
from google.protobuf import symbol_database as _symbol_database
from google.protobuf import descriptor_pb2
# @@protoc_insertion_point(imports)

_sym_db = _symbol_database.Default()




DESCRIPTOR = _descriptor.FileDescriptor(
  name='transactions_filter_stream.proto',
  package='org.dash.platform.dapi.v0',
  syntax='proto3',
  serialized_pb=_b('\n transactions_filter_stream.proto\x12\x19org.dash.platform.dapi.v0\"\xd3\x01\n\x1dTransactionsWithProofsRequest\x12<\n\x0c\x62loom_filter\x18\x01 \x01(\x0b\x32&.org.dash.platform.dapi.v0.BloomFilter\x12\x19\n\x0f\x66rom_block_hash\x18\x02 \x01(\x0cH\x00\x12\x1b\n\x11\x66rom_block_height\x18\x03 \x01(\rH\x00\x12\r\n\x05\x63ount\x18\x04 \x01(\r\x12\x1f\n\x17send_transaction_hashes\x18\x05 \x01(\x08\x42\x0c\n\nfrom_block\"U\n\x0b\x42loomFilter\x12\x0e\n\x06v_data\x18\x01 \x01(\x0c\x12\x14\n\x0cn_hash_funcs\x18\x02 \x01(\r\x12\x0f\n\x07n_tweak\x18\x03 \x01(\r\x12\x0f\n\x07n_flags\x18\x04 \x01(\r\"\xeb\x01\n\x1eTransactionsWithProofsResponse\x12\x46\n\x10raw_transactions\x18\x01 \x01(\x0b\x32*.org.dash.platform.dapi.v0.RawTransactionsH\x00\x12X\n\x1ainstant_send_lock_messages\x18\x02 \x01(\x0b\x32\x32.org.dash.platform.dapi.v0.InstantSendLockMessagesH\x00\x12\x1a\n\x10raw_merkle_block\x18\x03 \x01(\x0cH\x00\x42\x0b\n\tresponses\"\'\n\x0fRawTransactions\x12\x14\n\x0ctransactions\x18\x01 \x03(\x0c\"+\n\x17InstantSendLockMessages\x12\x10\n\x08messages\x18\x01 \x03(\x0c\x32\xb7\x01\n\x18TransactionsFilterStream\x12\x9a\x01\n!subscribeToTransactionsWithProofs\x12\x38.org.dash.platform.dapi.v0.TransactionsWithProofsRequest\x1a\x39.org.dash.platform.dapi.v0.TransactionsWithProofsResponse0\x01\x62\x06proto3')
)




_TRANSACTIONSWITHPROOFSREQUEST = _descriptor.Descriptor(
  name='TransactionsWithProofsRequest',
  full_name='org.dash.platform.dapi.v0.TransactionsWithProofsRequest',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  fields=[
    _descriptor.FieldDescriptor(
      name='bloom_filter', full_name='org.dash.platform.dapi.v0.TransactionsWithProofsRequest.bloom_filter', index=0,
      number=1, type=11, cpp_type=10, label=1,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None),
    _descriptor.FieldDescriptor(
      name='from_block_hash', full_name='org.dash.platform.dapi.v0.TransactionsWithProofsRequest.from_block_hash', index=1,
      number=2, type=12, cpp_type=9, label=1,
      has_default_value=False, default_value=_b(""),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None),
    _descriptor.FieldDescriptor(
      name='from_block_height', full_name='org.dash.platform.dapi.v0.TransactionsWithProofsRequest.from_block_height', index=2,
      number=3, type=13, cpp_type=3, label=1,
      has_default_value=False, default_value=0,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None),
    _descriptor.FieldDescriptor(
      name='count', full_name='org.dash.platform.dapi.v0.TransactionsWithProofsRequest.count', index=3,
      number=4, type=13, cpp_type=3, label=1,
      has_default_value=False, default_value=0,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None),
    _descriptor.FieldDescriptor(
      name='send_transaction_hashes', full_name='org.dash.platform.dapi.v0.TransactionsWithProofsRequest.send_transaction_hashes', index=4,
      number=5, type=8, cpp_type=7, label=1,
      has_default_value=False, default_value=False,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
    _descriptor.OneofDescriptor(
      name='from_block', full_name='org.dash.platform.dapi.v0.TransactionsWithProofsRequest.from_block',
      index=0, containing_type=None, fields=[]),
  ],
  serialized_start=64,
  serialized_end=275,
)


_BLOOMFILTER = _descriptor.Descriptor(
  name='BloomFilter',
  full_name='org.dash.platform.dapi.v0.BloomFilter',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  fields=[
    _descriptor.FieldDescriptor(
      name='v_data', full_name='org.dash.platform.dapi.v0.BloomFilter.v_data', index=0,
      number=1, type=12, cpp_type=9, label=1,
      has_default_value=False, default_value=_b(""),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None),
    _descriptor.FieldDescriptor(
      name='n_hash_funcs', full_name='org.dash.platform.dapi.v0.BloomFilter.n_hash_funcs', index=1,
      number=2, type=13, cpp_type=3, label=1,
      has_default_value=False, default_value=0,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None),
    _descriptor.FieldDescriptor(
      name='n_tweak', full_name='org.dash.platform.dapi.v0.BloomFilter.n_tweak', index=2,
      number=3, type=13, cpp_type=3, label=1,
      has_default_value=False, default_value=0,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None),
    _descriptor.FieldDescriptor(
      name='n_flags', full_name='org.dash.platform.dapi.v0.BloomFilter.n_flags', index=3,
      number=4, type=13, cpp_type=3, label=1,
      has_default_value=False, default_value=0,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=277,
  serialized_end=362,
)


_TRANSACTIONSWITHPROOFSRESPONSE = _descriptor.Descriptor(
  name='TransactionsWithProofsResponse',
  full_name='org.dash.platform.dapi.v0.TransactionsWithProofsResponse',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  fields=[
    _descriptor.FieldDescriptor(
      name='raw_transactions', full_name='org.dash.platform.dapi.v0.TransactionsWithProofsResponse.raw_transactions', index=0,
      number=1, type=11, cpp_type=10, label=1,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None),
    _descriptor.FieldDescriptor(
      name='instant_send_lock_messages', full_name='org.dash.platform.dapi.v0.TransactionsWithProofsResponse.instant_send_lock_messages', index=1,
      number=2, type=11, cpp_type=10, label=1,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None),
    _descriptor.FieldDescriptor(
      name='raw_merkle_block', full_name='org.dash.platform.dapi.v0.TransactionsWithProofsResponse.raw_merkle_block', index=2,
      number=3, type=12, cpp_type=9, label=1,
      has_default_value=False, default_value=_b(""),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
    _descriptor.OneofDescriptor(
      name='responses', full_name='org.dash.platform.dapi.v0.TransactionsWithProofsResponse.responses',
      index=0, containing_type=None, fields=[]),
  ],
  serialized_start=365,
  serialized_end=600,
)


_RAWTRANSACTIONS = _descriptor.Descriptor(
  name='RawTransactions',
  full_name='org.dash.platform.dapi.v0.RawTransactions',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  fields=[
    _descriptor.FieldDescriptor(
      name='transactions', full_name='org.dash.platform.dapi.v0.RawTransactions.transactions', index=0,
      number=1, type=12, cpp_type=9, label=3,
      has_default_value=False, default_value=[],
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=602,
  serialized_end=641,
)


_INSTANTSENDLOCKMESSAGES = _descriptor.Descriptor(
  name='InstantSendLockMessages',
  full_name='org.dash.platform.dapi.v0.InstantSendLockMessages',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  fields=[
    _descriptor.FieldDescriptor(
      name='messages', full_name='org.dash.platform.dapi.v0.InstantSendLockMessages.messages', index=0,
      number=1, type=12, cpp_type=9, label=3,
      has_default_value=False, default_value=[],
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=643,
  serialized_end=686,
)

_TRANSACTIONSWITHPROOFSREQUEST.fields_by_name['bloom_filter'].message_type = _BLOOMFILTER
_TRANSACTIONSWITHPROOFSREQUEST.oneofs_by_name['from_block'].fields.append(
  _TRANSACTIONSWITHPROOFSREQUEST.fields_by_name['from_block_hash'])
_TRANSACTIONSWITHPROOFSREQUEST.fields_by_name['from_block_hash'].containing_oneof = _TRANSACTIONSWITHPROOFSREQUEST.oneofs_by_name['from_block']
_TRANSACTIONSWITHPROOFSREQUEST.oneofs_by_name['from_block'].fields.append(
  _TRANSACTIONSWITHPROOFSREQUEST.fields_by_name['from_block_height'])
_TRANSACTIONSWITHPROOFSREQUEST.fields_by_name['from_block_height'].containing_oneof = _TRANSACTIONSWITHPROOFSREQUEST.oneofs_by_name['from_block']
_TRANSACTIONSWITHPROOFSRESPONSE.fields_by_name['raw_transactions'].message_type = _RAWTRANSACTIONS
_TRANSACTIONSWITHPROOFSRESPONSE.fields_by_name['instant_send_lock_messages'].message_type = _INSTANTSENDLOCKMESSAGES
_TRANSACTIONSWITHPROOFSRESPONSE.oneofs_by_name['responses'].fields.append(
  _TRANSACTIONSWITHPROOFSRESPONSE.fields_by_name['raw_transactions'])
_TRANSACTIONSWITHPROOFSRESPONSE.fields_by_name['raw_transactions'].containing_oneof = _TRANSACTIONSWITHPROOFSRESPONSE.oneofs_by_name['responses']
_TRANSACTIONSWITHPROOFSRESPONSE.oneofs_by_name['responses'].fields.append(
  _TRANSACTIONSWITHPROOFSRESPONSE.fields_by_name['instant_send_lock_messages'])
_TRANSACTIONSWITHPROOFSRESPONSE.fields_by_name['instant_send_lock_messages'].containing_oneof = _TRANSACTIONSWITHPROOFSRESPONSE.oneofs_by_name['responses']
_TRANSACTIONSWITHPROOFSRESPONSE.oneofs_by_name['responses'].fields.append(
  _TRANSACTIONSWITHPROOFSRESPONSE.fields_by_name['raw_merkle_block'])
_TRANSACTIONSWITHPROOFSRESPONSE.fields_by_name['raw_merkle_block'].containing_oneof = _TRANSACTIONSWITHPROOFSRESPONSE.oneofs_by_name['responses']
DESCRIPTOR.message_types_by_name['TransactionsWithProofsRequest'] = _TRANSACTIONSWITHPROOFSREQUEST
DESCRIPTOR.message_types_by_name['BloomFilter'] = _BLOOMFILTER
DESCRIPTOR.message_types_by_name['TransactionsWithProofsResponse'] = _TRANSACTIONSWITHPROOFSRESPONSE
DESCRIPTOR.message_types_by_name['RawTransactions'] = _RAWTRANSACTIONS
DESCRIPTOR.message_types_by_name['InstantSendLockMessages'] = _INSTANTSENDLOCKMESSAGES
_sym_db.RegisterFileDescriptor(DESCRIPTOR)

TransactionsWithProofsRequest = _reflection.GeneratedProtocolMessageType('TransactionsWithProofsRequest', (_message.Message,), dict(
  DESCRIPTOR = _TRANSACTIONSWITHPROOFSREQUEST,
  __module__ = 'transactions_filter_stream_pb2'
  # @@protoc_insertion_point(class_scope:org.dash.platform.dapi.v0.TransactionsWithProofsRequest)
  ))
_sym_db.RegisterMessage(TransactionsWithProofsRequest)

BloomFilter = _reflection.GeneratedProtocolMessageType('BloomFilter', (_message.Message,), dict(
  DESCRIPTOR = _BLOOMFILTER,
  __module__ = 'transactions_filter_stream_pb2'
  # @@protoc_insertion_point(class_scope:org.dash.platform.dapi.v0.BloomFilter)
  ))
_sym_db.RegisterMessage(BloomFilter)

TransactionsWithProofsResponse = _reflection.GeneratedProtocolMessageType('TransactionsWithProofsResponse', (_message.Message,), dict(
  DESCRIPTOR = _TRANSACTIONSWITHPROOFSRESPONSE,
  __module__ = 'transactions_filter_stream_pb2'
  # @@protoc_insertion_point(class_scope:org.dash.platform.dapi.v0.TransactionsWithProofsResponse)
  ))
_sym_db.RegisterMessage(TransactionsWithProofsResponse)

RawTransactions = _reflection.GeneratedProtocolMessageType('RawTransactions', (_message.Message,), dict(
  DESCRIPTOR = _RAWTRANSACTIONS,
  __module__ = 'transactions_filter_stream_pb2'
  # @@protoc_insertion_point(class_scope:org.dash.platform.dapi.v0.RawTransactions)
  ))
_sym_db.RegisterMessage(RawTransactions)

InstantSendLockMessages = _reflection.GeneratedProtocolMessageType('InstantSendLockMessages', (_message.Message,), dict(
  DESCRIPTOR = _INSTANTSENDLOCKMESSAGES,
  __module__ = 'transactions_filter_stream_pb2'
  # @@protoc_insertion_point(class_scope:org.dash.platform.dapi.v0.InstantSendLockMessages)
  ))
_sym_db.RegisterMessage(InstantSendLockMessages)



_TRANSACTIONSFILTERSTREAM = _descriptor.ServiceDescriptor(
  name='TransactionsFilterStream',
  full_name='org.dash.platform.dapi.v0.TransactionsFilterStream',
  file=DESCRIPTOR,
  index=0,
  options=None,
  serialized_start=689,
  serialized_end=872,
  methods=[
  _descriptor.MethodDescriptor(
    name='subscribeToTransactionsWithProofs',
    full_name='org.dash.platform.dapi.v0.TransactionsFilterStream.subscribeToTransactionsWithProofs',
    index=0,
    containing_service=None,
    input_type=_TRANSACTIONSWITHPROOFSREQUEST,
    output_type=_TRANSACTIONSWITHPROOFSRESPONSE,
    options=None,
  ),
])
_sym_db.RegisterServiceDescriptor(_TRANSACTIONSFILTERSTREAM)

DESCRIPTOR.services_by_name['TransactionsFilterStream'] = _TRANSACTIONSFILTERSTREAM

try:
  # THESE ELEMENTS WILL BE DEPRECATED.
  # Please use the generated *_pb2_grpc.py files instead.
  import grpc
  from grpc.beta import implementations as beta_implementations
  from grpc.beta import interfaces as beta_interfaces
  from grpc.framework.common import cardinality
  from grpc.framework.interfaces.face import utilities as face_utilities


  class TransactionsFilterStreamStub(object):
    # missing associated documentation comment in .proto file
    pass

    def __init__(self, channel):
      """Constructor.

      Args:
        channel: A grpc.Channel.
      """
      self.subscribeToTransactionsWithProofs = channel.unary_stream(
          '/org.dash.platform.dapi.v0.TransactionsFilterStream/subscribeToTransactionsWithProofs',
          request_serializer=TransactionsWithProofsRequest.SerializeToString,
          response_deserializer=TransactionsWithProofsResponse.FromString,
          )


  class TransactionsFilterStreamServicer(object):
    # missing associated documentation comment in .proto file
    pass

    def subscribeToTransactionsWithProofs(self, request, context):
      # missing associated documentation comment in .proto file
      pass
      context.set_code(grpc.StatusCode.UNIMPLEMENTED)
      context.set_details('Method not implemented!')
      raise NotImplementedError('Method not implemented!')


  def add_TransactionsFilterStreamServicer_to_server(servicer, server):
    rpc_method_handlers = {
        'subscribeToTransactionsWithProofs': grpc.unary_stream_rpc_method_handler(
            servicer.subscribeToTransactionsWithProofs,
            request_deserializer=TransactionsWithProofsRequest.FromString,
            response_serializer=TransactionsWithProofsResponse.SerializeToString,
        ),
    }
    generic_handler = grpc.method_handlers_generic_handler(
        'org.dash.platform.dapi.v0.TransactionsFilterStream', rpc_method_handlers)
    server.add_generic_rpc_handlers((generic_handler,))


  class BetaTransactionsFilterStreamServicer(object):
    """The Beta API is deprecated for 0.15.0 and later.

    It is recommended to use the GA API (classes and functions in this
    file not marked beta) for all further purposes. This class was generated
    only to ease transition from grpcio<0.15.0 to grpcio>=0.15.0."""
    # missing associated documentation comment in .proto file
    pass
    def subscribeToTransactionsWithProofs(self, request, context):
      # missing associated documentation comment in .proto file
      pass
      context.code(beta_interfaces.StatusCode.UNIMPLEMENTED)


  class BetaTransactionsFilterStreamStub(object):
    """The Beta API is deprecated for 0.15.0 and later.

    It is recommended to use the GA API (classes and functions in this
    file not marked beta) for all further purposes. This class was generated
    only to ease transition from grpcio<0.15.0 to grpcio>=0.15.0."""
    # missing associated documentation comment in .proto file
    pass
    def subscribeToTransactionsWithProofs(self, request, timeout, metadata=None, with_call=False, protocol_options=None):
      # missing associated documentation comment in .proto file
      pass
      raise NotImplementedError()


  def beta_create_TransactionsFilterStream_server(servicer, pool=None, pool_size=None, default_timeout=None, maximum_timeout=None):
    """The Beta API is deprecated for 0.15.0 and later.

    It is recommended to use the GA API (classes and functions in this
    file not marked beta) for all further purposes. This function was
    generated only to ease transition from grpcio<0.15.0 to grpcio>=0.15.0"""
    request_deserializers = {
      ('org.dash.platform.dapi.v0.TransactionsFilterStream', 'subscribeToTransactionsWithProofs'): TransactionsWithProofsRequest.FromString,
    }
    response_serializers = {
      ('org.dash.platform.dapi.v0.TransactionsFilterStream', 'subscribeToTransactionsWithProofs'): TransactionsWithProofsResponse.SerializeToString,
    }
    method_implementations = {
      ('org.dash.platform.dapi.v0.TransactionsFilterStream', 'subscribeToTransactionsWithProofs'): face_utilities.unary_stream_inline(servicer.subscribeToTransactionsWithProofs),
    }
    server_options = beta_implementations.server_options(request_deserializers=request_deserializers, response_serializers=response_serializers, thread_pool=pool, thread_pool_size=pool_size, default_timeout=default_timeout, maximum_timeout=maximum_timeout)
    return beta_implementations.server(method_implementations, options=server_options)


  def beta_create_TransactionsFilterStream_stub(channel, host=None, metadata_transformer=None, pool=None, pool_size=None):
    """The Beta API is deprecated for 0.15.0 and later.

    It is recommended to use the GA API (classes and functions in this
    file not marked beta) for all further purposes. This function was
    generated only to ease transition from grpcio<0.15.0 to grpcio>=0.15.0"""
    request_serializers = {
      ('org.dash.platform.dapi.v0.TransactionsFilterStream', 'subscribeToTransactionsWithProofs'): TransactionsWithProofsRequest.SerializeToString,
    }
    response_deserializers = {
      ('org.dash.platform.dapi.v0.TransactionsFilterStream', 'subscribeToTransactionsWithProofs'): TransactionsWithProofsResponse.FromString,
    }
    cardinalities = {
      'subscribeToTransactionsWithProofs': cardinality.Cardinality.UNARY_STREAM,
    }
    stub_options = beta_implementations.stub_options(host=host, metadata_transformer=metadata_transformer, request_serializers=request_serializers, response_deserializers=response_deserializers, thread_pool=pool, thread_pool_size=pool_size)
    return beta_implementations.dynamic_stub(channel, 'org.dash.platform.dapi.v0.TransactionsFilterStream', cardinalities, options=stub_options)
except ImportError:
  pass
# @@protoc_insertion_point(module_scope)



import grpc
import cbor2

# Generated from dapi-grpc
import dapiclient.rpc.grpc.core_pb2 as core_pb2
import dapiclient.rpc.grpc.core_pb2_grpc as core_pb2_grpc
import dapiclient.rpc.grpc.platform_pb2 as platform_pb2
import dapiclient.rpc.grpc.platform_pb2_grpc as platform_pb2_grpc
import dapiclient.rpc.grpc.transactions_filter_stream_pb2 as transactions_filter_stream_pb2
import dapiclient.rpc.grpc.transactions_filter_stream_pb2_grpc as transactions_filter_stream_pb2_grpc


class GRpcClient:

    def __init__(self):
        pass

    def request(socket, method, params = {}, options = {'timeout': 5}):
        channel = grpc.insecure_channel(socket, options=(('grpc.enable_http_proxy', 0),))
        #print(socket)

        stub = platform_pb2_grpc.PlatformStub(channel)
        stubCore = core_pb2_grpc.CoreStub(channel)
        stubTransactions = transactions_filter_stream_pb2_grpc.TransactionsFilterStreamStub(channel)

        if method == 'getIdentity':
            return GRpcClient.getIdentity(stub, params, options)

        elif method == 'getDataContract':
            return GRpcClient.getDataContract(stub, params, options)

        elif method == 'getDocuments':
            return GRpcClient.getDocuments(stub, params, options)

        elif method == 'applyStateTransition':
            return GRpcClient.applyStateTransition(stub, params, options)

        elif method == 'getIdentityByFirstPublicKey':
            return GRpcClient.getIdentityByFirstPublicKey(stub, params, options)

        elif method == 'getIdentityIdByFirstPublicKey':
            return GRpcClient.getIdentityIdByFirstPublicKey(stub, params, options)

        elif method == 'getBlock':
            return GRpcClient.getBlock(stubCore, params, options)

        elif method == 'getStatus':
            return GRpcClient.getStatus(stubCore, params, options)

        elif method == 'getTransaction':
            return GRpcClient.getTransaction(stubCore, params, options)

        elif method == 'sendTransaction':
            return GRpcClient.sendTransaction(stubCore, params, options)

        elif method == 'subscribeToTransactionsWithProofs':
            return GRpcClient.subscribeToTransactionsWithProofs(stubTransactions, params, options)

        else:
            raise ValueError('Unknown gRPC endpoint: {}'.format(method))


    def getIdentity(stub, params, options):
        # Create Identity request
        identity_request = platform_pb2.GetIdentityRequest()
        # Set identity parameter of request
        identity_request.id = params['id']
       
        response = stub.getIdentity(identity_request, options['timeout'])

        responseBytes = bytearray(response.identity)
        identityBytes = responseBytes[4 : len(responseBytes)]


        return cbor2.loads(identityBytes)


    def getDataContract(stub, params, options):
        contract_request = platform_pb2.GetDataContractRequest()
        contract_request.id = params['id']

        response = stub.getDataContract(contract_request, options['timeout'])
        #print('Data Contract: {}\n'.format(cbor2.loads(response.data_contract)))

        return cbor2.loads(response.data_contract)


    def getDocuments(stub, params, options):
        #print(params)

        document_request = platform_pb2.GetDocumentsRequest()
        document_request.data_contract_id = params['data_contract_id']
        document_request.document_type =  params['document_type']
        document_request.where = params['where']
        document_request.order_by = params['order_by']
        document_request.limit =  params['limit']
        document_request.start_at =  params['start_at']
        document_request.start_after = params['start_after']

        response = stub.getDocuments(document_request, options['timeout'])

        documents = []
        for doc in response.documents:
             docBytes = doc[4 : len(doc)]
             documents.append(cbor2.loads(docBytes))
             
        return documents

    def getBlock(stubCore, params, options):
        block_request = core_pb2.GetBlockRequest()
        block_request.hash = params['hash']
        block_request.height = params['height']

        response = stubCore.getBlock(block_request, options['timeout'])
        return cbor2.loads(response.block)

    def getStatus(stubCore, params, options):
        status_request = core_pb2.GetStatusRequest()

        response = stubCore.getStatus(status_request, options['timeout'])

        return response

    def getTransaction(stubCore, params, options):
        transaction_request = core_pb2.GetTransactionRequest()
        transaction_request.id = params['id']

        response = stubCore.getTransaction(transaction_request, options['timeout'])

        return response.transaction

    def sendTransaction(stubCore, params, options):
        transaction_request = core_pb2.SendTransactionRequest()
        transaction_request.transaction = params['transaction']
        transaction_request.allow_high_fees = params['allow_high_fees']
        transaction_request.bypass_limits = params['bypass_limits']

        response = stubCore.sendTransaction(transaction_request, options['timeout'])

        return response

    def subscribeToTransactionsWithProofs(stubTransactions, params, options):
        subscribe_request = transactions_filter_stream_pb2.TransactionsWithProofsRequest()
        subscribe_request.from_block_hash = params['from_block_hash']
        subscribe_request.from_block_height = params['from_block_height']
        subscribe_request.count = params['count']
        subscribe_request.send_transaction_hashes = params['send_transaction_hashes']
        setattr(subscribe_request.bloom_filter, 'n_hash_funcs', params['bloom_filter']['n_hash_funcs'])
        setattr(subscribe_request.bloom_filter, 'v_data', params['bloom_filter']['v_data'])
        setattr(subscribe_request.bloom_filter, 'n_tweak', params['bloom_filter']['n_tweak'])
        setattr(subscribe_request.bloom_filter, 'n_flags', params['bloom_filter']['n_flags'])

        response = stubTransactions.transactionWithProof(subscribe_request, options['timeout'])

        return response

    def applyStateTransition(stub, params, options):
        apply_request = platform_pb2.ApplyStateTransitionRequest()
        apply_request.state_transition = params['state_transition']

        response = stub.applyStateTransition(apply_request, options['timeout'])

        return response

    def getIdentityByFirstPublicKey(stub, params, options):
        request = platform_pb2.GetIdentityByFirstPublicKeyRequest()
        request.public_key_hash = params['public_key_hash']

        response = stub.getIdentityByFirstPublicKey(request, options['timeout'])

        return response

    def getIdentityIdByFirstPublicKey(stub, params, options):
        request = platform_pb2.GetIdentityIdByFirstPublicKeyRequest()
        request.public_key_hash = params['public_key_hash']

        response = stub.getIdentityIdByFirstPublicKey(request, options['timeout'])

        return response



import random
import datetime
from dapiclient.rpc.jsonrpc.jsonrpc_client import JsonRpcClient

# TODO: Stop using hard-coded seed in all requests
SEED_IP = 'seed-1.testnet.networks.dash.org'
SEED_PORT = 3000
MN_LIST_UPDATE_INTERVAL = 60000 / 1000



'''
 * This class holds the valid deterministic masternode list
 * @type {SimplifiedMNListEntry[]}
 * @property {string} proRegTxHash
 * @property {string} confirmedHash
 * @property {string} service - ip and port
 * @property {string} pubKeyOperator - operator public key
 * @property {string} keyIDVoting - public key hash, 20 bytes
 * @property {boolean} isValid
'''
class SimplifiedMNListEntry:
    def __init__(self, seeds, dapi_port = 3000):
        #self.mn_service = set()
        self.mn_entry = []

    def add_entry(self, entry):
        self.mn_entry.append(entry)

    #def add_service(self, service):
    #    self.mn_service.add(service)

    def get_random_masternode(self):
        return random.sample(self.mn_entry, 1)[0]['service']

class MasternodeListProvider:
    def __init__(self, seeds, dapi_port = SEED_PORT):
        self.SEED_IP = seeds
        if seeds is None:
            seeds = SEED_IP
        if dapi_port is None:
            dapi_port = SEED_PORT

        self.masternode_list = []
        #self.simplified_masternode_list = SimplifiedMNList()
        self.dapi_port = dapi_port
        self.last_update_date = 0
        self.base_block_hash = '0000000000000000000000000000000000000000000000000000000000000000' #constants.masternodeList.NULL_HASH;

    def isEmptyMasternodeList(self):
        return True if len(self.masternode_list.length) == 0 else False


    def get_genesis_hash(self):
        genesis_height = 0
        #node = self.isEmptyMasternodeList() ? sample(self.seeds) : sample(self.masternodeList);
        #ip_address = node.service.split(':')[0];
        ip_address = self.SEED_IP

        genesis_block_hash = JsonRpcClient.request({
            'host': ip_address,
            'port': self.dapi_port
        }, 'getBlockHash', { 'height': genesis_height })
        return genesis_block_hash


    def update_mn_list(self):
    #if (self.base_block_hash === config.nullHash) {
        self.base_block_hash = self.get_genesis_hash()
        diff = self.get_simplified_mn_list_diff()

        #self.masternode_list = validMasternodesList;
        self.masternode_list = diff['mnList']
        self.base_block_hash = diff['baseBlockHash']
        self.last_update_date = datetime.datetime.utcnow().timestamp()


    def get_simplified_mn_list_diff(self):
        #node = this.isEmptyMasternodeList() ? sample(this.seeds) : sample(this.masternodeList);
        base_block_hash = self.base_block_hash;
        ip_address =  self.SEED_IP #node.service.split(':')[0];

        block_hash = JsonRpcClient.request({
            'host': ip_address,
            'port': self.dapi_port
        }, 'getBestBlockHash')

        # TODO: error checking on blockhash response

        #if (!blockHash) {
        #  throw new Error(`Failed to get best block hash for getSimplifiedMNListDiff from node ${ipAddress}`);
        #}

        params = {
            'baseBlockHash': base_block_hash,
            'blockHash': block_hash
        }
        diff = JsonRpcClient.request({
            'host': ip_address,
            'port': self.dapi_port
        }, 'getMnListDiff', params)

        # TODO: error checking on diff response

        #if (!diff) {
        #  throw new Error(`Failed to get mn diff from node ${ipAddress}`);
        #}
        return diff


    def needs_update(self):
        time_since_update = datetime.datetime.utcnow().timestamp() - self.last_update_date #MN_LIST_UPDATE_INTERVAL
        #print('Time since update: {}; Limit: {}'.format(time_since_update, MN_LIST_UPDATE_INTERVAL))
        return True if time_since_update > MN_LIST_UPDATE_INTERVAL else False


    def get_mn_list(self):
        if (self.needs_update()):
            #print('Masternode List needs updating...')
            self.update_mn_list()

        return self.masternode_list


    def check_mn_response(self, ip_address):
        try:
            block_hash = JsonRpcClient.request({
                'host': ip_address,
                'port': self.dapi_port
            }, 'getBestBlockHash')
            print('Success from {}:\t{}'.format(ip_address, block_hash))
        except Exception as ex:
            print('Failure from {}:\t** {} **'.format(ip_address, ex))


def main():
    mnlist_provider = MasternodeListProvider(None)
    masternode_list = mnlist_provider.get_mn_list()

    print('Masternode list contain {} masternodes'.format(len(masternode_list)))

    print('Checking JSON-RPC response from all masternodes')
    for m in masternode_list:
        mnlist_provider.check_mn_response(m['service'].split(':')[0])
    return

if __name__ == "__main__":
    main()
